import os
import tempfile
import unittest
from unittest import TestCase


from main import settings

from system import bpio

from logs import lg

from lib import strng

from userid import id_url
from userid import identity


alice_bin = b'http://127.0.0.1:8084/alice.xml'
alice_text = 'http://127.0.0.1:8084/alice.xml'
bob = 'http://127.0.0.1/bob.xml'
carl = 'http://127.0.0.1/carl.xml'
dave = b'http://127.0.0.1:8084/dave.xml'
ethan_bin = b'http://127.0.0.1:8084/ethan.xml'
ethan_text = 'http://127.0.0.1:8084/ethan.xml'
ethan_not_exist = 'http://not-exist.com/ethan.xml'
frank_1 = 'http://source_one.com/frank.xml'
frank_2 = 'http://source_two.net/frank.xml'
fake_frank = 'http://source_one_fake.com/frank.xml'
george_1 = 'http://source_one.com/george.xml'
george_2 = 'http://source_two.net/george.xml'
george_3 = 'http://source_3.org/george.xml'
hans1 = 'http://first.com/hans.xml'
hans2 = 'http://second.net/hans.xml'
hans3 = 'http://third.org/hans.xml'


class TestIDURL(TestCase):

    def setUp(self):
        try:
            bpio.rmdir_recursive('/tmp/.bitdust_tmp')
        except Exception:
            pass
        lg.set_debug_level(30)
        settings.init(base_dir='/tmp/.bitdust_tmp')
        id_url._IdentityHistoryDir = tempfile.mkdtemp()
        id_url.init()
        try:
            os.makedirs('/tmp/.bitdust_tmp/identitycache/')
        except:
            pass
        try:
            os.makedirs('/tmp/.bitdust_tmp/identityhistory/')
        except:
            pass

    def tearDown(self):
        id_url.shutdown()
        settings.shutdown()
        bpio.rmdir_recursive('/tmp/.bitdust_tmp')

    def _cache_identity(self, idname='alice'):
        known = {
            'alice': """<?xml version="1.0" encoding="utf-8"?>
                <identity>
                  <sources>
                    <source>http://127.0.0.1:8084/alice.xml</source>
                  </sources>
                  <contacts>
                    <contact>tcp://127.0.0.1:7103</contact>
                  </contacts>
                  <certificates/>
                  <scrubbers/>
                  <postage>0</postage>
                  <date>Oct 06, 2018</date>
                  <version></version>
                  <revision>0</revision>
                  <publickey>ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQD9mwkrIJqSoDy87avQJM4bSoTaX7jLV0iqHtULShpWRfLQKKazc34023q8nrXsfkV4wAtqxF/j7mceSV5MH4EpMoHG3h8ub0zPeaTDxT6my0l9jNIWOMbdmOzpcLgfIyM4JORgJ29BBN6oH0AsAnXRilXCUmVc5oWixzC/cuRQPhcHZDiuHqKph1Cjs1ra9jRoDD63/BOctPOv+vFC6FueDuS0+/xlTHr14fMbTuAtGmrNgfvOS+6KO16zE8gUUtpKWBA9zesgurAs//Ajp8OO09aWcbVwkn17s8GrUuRDwUmgLFpLH3/OtlxW4oWcI8atzWGOoIhT0eHfBM/FX88h</publickey>
                  <signature>906441963064925827454594808955119786427327091644004203121255573673324339896720684156436969809331302081059171001468697744607176215971850395448358447221987030255681283469602485017439100048818715591137645334649289784704347547476744833593959051239987455656492991690929110057776187651745011046495572223232179906271885029639566994962961751420295596050170876718201976232073313442092469148257068924122205928495395843354406033087633663157320015417942886925426409641297819144861516715728591303834175552581452070473350075769438068777969720417764064190628218036180170101856467748070658740635441921977032005183554438715223073852</signature>
                </identity>""",
            'bob': """<?xml version="1.0" encoding="utf-8"?>
                <identity>
                  <sources>
                    <source>http://127.0.0.1/bob.xml</source>
                  </sources>
                  <contacts>
                    <contact>tcp://:7592</contact>
                  </contacts>
                  <certificates/>
                  <scrubbers/>
                  <postage>1</postage>
                  <date>May 29, 2019</date>
                  <version></version>
                  <revision>0</revision>
                  <publickey>ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQCewFs0xQAgdUXI0tzgWco6I4m47UCwvQTXs0fwF3gH+x+iAgx2RgTvjFpURyZPK45IrfF1kWvAE66ztzy2j+6hhHBXw58M3mj8zquDEKa+4mZFZdUtYRm6mNR7CfC9Nbot6wV52bHxsiLUxQvbBnc57lY/eMErXZSbUJjTjryCfVn7t5RS5aALQBfIVMfNMaVCJ3e5Rye0dhR6T9ZFjrLkReuo0okpCvoZZOkDIP1jLCjcOTv8uLUv+drqf8NIrZIj4pEF8rdRNujaocfiNThgkCBdmIhVrgkfIdK3vWM6QhJ3kZtulth1tDV5BK8/lfFZaPxfFGweR1/Gg9QDpOD9</publickey>
                  <signature>2555721207758289999127178087506098515147593088396681387272515731590789858885100168022894000811640385932332602114921724686816736386919493342116388342389202634533573440585853004416336080009871348355034060900570108740776929249059366125313043773781172121438059275318503578278037211137723742166236250810224379701898784507989109650122280574248889624793623753363750956166337580150155645791129345184984075393339236370769934887288067282063889985832781680930818596333533250385870238328128795908788457961092966872255919485411937392364346156912286424098295431775553440859616188700512071957957128388244692261317739667793487752010</signature>
                </identity>""",
            'carl': """<?xml version="1.0" encoding="utf-8"?>
                <identity>
                  <sources>
                    <source>http://127.0.0.1/carl.xml</source>
                  </sources>
                  <contacts>
                    <contact>tcp://:7592</contact>
                  </contacts>
                  <certificates/>
                  <scrubbers/>
                  <postage>1</postage>
                  <date>May 29, 2019</date>
                  <version></version>
                  <revision>0</revision>
                  <publickey>ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQC36qNFejdTYd1GhNY8OW4Qyl5T9invqwF9/4AIjQX2r2bpqUj6imw+3H+EtELezUGuPEE7HBN2CXVIkvfz19L4nyLtHgmEgS8urX6DZl33fRmoJ17kKzU/64c1zijo+XbKfbAoEqWnY3PSUSDPSjOCYCAfLweVc6iwAK50Spoi8i5jh4qu6Oo5Suk9JU3IPNyoCY0jwxyG2pXOF5UKPaZLZnpTkbaiLINGlBHrOR+6pfIt0ab1G1IAOdz7ylakRDIiOLqWT6q8vgWyYFilSk3wb/iSu5oXfwSY7lLJlMY1xMz+gK+O7EyJ8aoxQGVZKmheNAQI0iQe8K0CVYepzsmv</publickey>
                  <signature>5671990383316217799860037685304343495387058906806556711839691386733274913975789513413370897454962593040260737131290348634715962286536496049223361697406163016265431897815056269698855557291580855474900681189364651092579128930585955713599705768551842925654460065820160273829629149488996975869038456697474472203533938277041514807958092108274858796695674035784529088500884603138043464155211422881469779643934748226157422190975487363378050151604507805760469094648075872918761427451548300297228184991006257005174164375453061839910123595693965585658191650111516102856331851331452134501115435806747753564318702409801372882796</signature>
                </identity>""",
            'frank': """<?xml version="1.0" encoding="utf-8"?>
                <identity>
                  <sources>
                    <source>http://source_one.com/frank.xml</source>
                    <source>http://source_two.net/frank.xml</source>
                  </sources>
                  <contacts>
                    <contact>tcp://:7592</contact>
                  </contacts>
                  <certificates/>
                  <scrubbers/>
                  <postage>1</postage>
                  <date>May 30, 2019</date>
                  <version></version>
                  <revision>0</revision>
                  <publickey>ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQC/nDAJOYUyFnT5lVjwKPNYUGW5NJYQvzi8wautEPUG8ugN1ZkbRnZBJ46pJwaUIW1gUlJzL7bORi/Wy5tzAcvkWuFhDw+86w8Ex9r2X2Nrgr5cavueze11oOQ37SKFPDrZTYjhq2ed+rTSVJ42rylER9riAysJl1RAr6MsC7QtZ1KZDIWDzPQ7WQrvcoFVLK5o/paIsYLZcTccOALyS9CvXrNS9nYibU3Pyaq/zMHpFWY4nPAT7B0klyGiJzAHqTw1dNbAypXvnLVTG4eNKMbArXUpCHeDDY53tlnYkNDzuceB2lz86NpA2Ewx5Lr0w3/9N9yHsWgkYwNS/JfTD3bX</publickey>
                  <signature>1470399213647704633900402631360190748435234657805006286636791463584064835488129632598194044665351596044943525434240751740260636768475013148198222238722883426082432682475514476350120815733632509582355723720357735336589375729701597306429832373309354454899504698264706992923925941564707082145998420064958807862379138356373859695029687660005618278346183697586798708643920788468297695914337947578382893671765602095325079806610626195889332254501099459474380009351208434503202002631112285047914334438054000200851146620696476908095380446889155676867174774326398005266101164712932326179585975162203098025117398260191052011443</signature>
                </identity>""",
            'george': """<?xml version="1.0" encoding="utf-8"?>
                <identity>
                  <sources>
                    <source>http://source_one.com/george.xml</source>
                    <source>http://source_two.net/george.xml</source>
                    <source>http://source_3.org/george.xml</source>
                  </sources>
                  <contacts>
                    <contact>tcp://:7592</contact>
                  </contacts>
                  <certificates/>
                  <scrubbers/>
                  <postage>1</postage>
                  <date>May 30, 2019</date>
                  <version></version>
                  <revision>0</revision>
                  <publickey>ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQC/IiHo/Nczuyzkd+0x4CUrzIO0iQyBAOdQZ3SRfgECfsYFemS7oYqp2DoaCrkuMaXc5211woY4Mvq25UZ7DUsx5o+mDoYoxU7HlgEB2QQnZZS7ExEsI+Ea11aTjrPJfkV7RkM9dUoiS7OkyS4PpCTV+S/vAA3bfkigWyF/FDDUBZiR2Gt39nbuotMvzFAGsDCS6pvrg/6Zkvnt4nO9ArQ0fYnTz7vtL5So2vE4NOwja4afvnn2Va+bubHDobufXFfpsHckyIO7JWHXMHULM5CGxSjV3ou4cgsSFzE/LovraK8IYamIysW51JApCvNGHXvPPO56CtpWFtMf/jQQ8qjB</publickey>
                  <signature>4114766194913785844307623478268920513207904809649390295191759339791248788857035079837285467140015294884268810863059294414714558478252286050429857099922149496212946962529416109370546760242342495327383015158042933406471591968103744617569754084992294893819302190358783127462323722586367855113145523366763033558219064643619554495523421005392982238445015008467696933018619012777969351010689810426614808111374535389775850513090968536153491255480569519856623066891877017655355744483137731360961139683030816198494031343962402377957266866817307216410794084720444986031021965708218747085220015804173278724096171222791791667812</signature>
                </identity>""",
            'fake_frank': """<?xml version="1.0" encoding="utf-8"?>
                <identity>
                  <sources>
                    <source>http://source_one_fake.com/frank.xml</source>
                    <source>http://source_two.net/frank.xml</source>
                  </sources>
                  <contacts>
                    <contact>tcp://127.0.0.1:7169</contact>
                  </contacts>
                  <certificates/>
                  <scrubbers/>
                  <postage>1</postage>
                  <date>Jul 01, 2019</date>
                  <version></version>
                  <revision>0</revision>
                  <publickey>ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQCS5wf0kaJa7zLwYWBEj+/JzfMhxFcMUy6Fz6DcuFCK05i1VgeFHuEEbpkU4yAYiNX1YGb4FBOfJ95UFM8HOB6b8QDqA6uHOkjT/aNiaL7fj+LG7UwqGef5cEWbckIpaeZQiyWaNuTtp2rywPrEakIs7KUFcdtiPhvtjymT8PyFRhSOoRO2U9+54K+FFJH9XQtSPB+KNdNOm18tFPJ4lviEODCKH0rE9BQ4Vn/auO0KBcwX8AGuhUFI0nsaQtHcty1L6u1UXHrEXGZeu2yBhEsHDHunxW+h0YOgCRdI8Usnobdx/dyJ7momABnu6mElt/ylHzM98nM/NfmM5BOrG9Tv</publickey>
                  <signature>13107260235651409913492390604656826449101578484743079806566807521296855989324803242511319779707838174946127035306858215504290452087950300644093221553306839853186323155012108941951146067164825000001482898225482538135328083540275907917650591073062847715060678819674667948896781152495669494916549386439852946976965970389941158492538138518594892617227070532550696583600653695158346473408694303871305287828335597832662071836974941142069312163761621166361732834315345454522029016656941848429213536700288288831888079827237495251089188669161987255910430438434187684912979732275126065406389681858156899388574380125659081087827</signature>
                </identity>""",
            'hans1': """<?xml version="1.0" encoding="utf-8"?>
                <identity>
                  <sources>
                    <source>http://first.com/hans.xml</source>
                    <source>http://second.net/hans.xml</source>
                  </sources>
                  <contacts>
                    <contact>tcp://127.0.0.1:7457</contact>
                  </contacts>
                  <certificates/>
                  <scrubbers/>
                  <postage>1</postage>
                  <date>Jul 01, 2019</date>
                  <version></version>
                  <revision>0</revision>
                  <publickey>ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQCiMX5AjFoK+B8bEts97OEKkJmONy8wDVSTe4Sx356p1fd48UQHq0g3xphfEWqZNVEvvXyVT3ToJZpsn6ZXALR6awp1EosV0Y3eCRn3HJ7VFifsObEBaJlbIpPWO3a44yQuNmB18dpAZsOYF0fuv9O9JZF/r2aS3DwJKKvrb1raPtuOkmLvMFOyFzQ4CzbpzOhxfLyk4VyyqWtxgRWa3cLJRC1s8pZP+Eeujz9lUXJOBJkz458myjcNogZ60HqMWPmNEQxKQKxKz5s1KhTzEa13AbK3mfBz6GYRSUE4PgzPNGt3ggKjm109MCECVLJ20i41l1x0LQogH4io0zN1KGFJ</publickey>
                  <signature>13068553383637753085388545974152093609980484012770452572120600195304903547295645115743544723314736720580808645983593697230309177682054712275065288665744892926477896400614384619512508972942866186572904043048207845632626832350683561290105271937200406689170558357079802162498698787054408426873656285478048378799191423141745937328344899208835186975874729426520455853791306586430419171014111851754881354985456443997095404651969419163775186405638979606072817375092764901667492456855939205354792829575562245938026302306124652445463109637611383925691836897970969979043999831425972117577943735372131767450971245110225874441582</signature>
                </identity>""",
            'hans2': """<?xml version="1.0" encoding="utf-8"?>
                <identity>
                  <sources>
                    <source>http://second.net/hans.xml</source>
                    <source>http://third.org/hans.xml</source>
                  </sources>
                  <contacts>
                    <contact>tcp://127.0.0.1:7457</contact>
                  </contacts>
                  <certificates/>
                  <scrubbers/>
                  <postage>1</postage>
                  <date>Jul 01, 2019</date>
                  <version></version>
                  <revision>1</revision>
                  <publickey>ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQCiMX5AjFoK+B8bEts97OEKkJmONy8wDVSTe4Sx356p1fd48UQHq0g3xphfEWqZNVEvvXyVT3ToJZpsn6ZXALR6awp1EosV0Y3eCRn3HJ7VFifsObEBaJlbIpPWO3a44yQuNmB18dpAZsOYF0fuv9O9JZF/r2aS3DwJKKvrb1raPtuOkmLvMFOyFzQ4CzbpzOhxfLyk4VyyqWtxgRWa3cLJRC1s8pZP+Eeujz9lUXJOBJkz458myjcNogZ60HqMWPmNEQxKQKxKz5s1KhTzEa13AbK3mfBz6GYRSUE4PgzPNGt3ggKjm109MCECVLJ20i41l1x0LQogH4io0zN1KGFJ</publickey>
                  <signature>9964338615595898119523219160985389694716834455244251121310208348749311239026480163448985744966067564091031002898262983039746088711129723160991957466541609144298294968214985017280673670405176798626865604720543551314153138295619813468551357860935622955238006750497782286078815952649485259190562480676412057686853832517927221963473242813486514373660489478033158129265672776156687967394550999847921296632829991543789469343181140623972591265599094214576144469741220772769413300453153162888368211327094417654762184709016011327148922218562411856167502006839171679742071717903072564535777785105712764567873880374805969385183</signature>
                </identity>""",
            'hans3': """<?xml version="1.0" encoding="utf-8"?>
                <identity>
                  <sources>
                    <source>http://third.org/hans.xml</source>
                  </sources>
                  <contacts>
                    <contact>tcp://127.0.0.1:7457</contact>
                  </contacts>
                  <certificates/>
                  <scrubbers/>
                  <postage>1</postage>
                  <date>Jul 01, 2019</date>
                  <version></version>
                  <revision>2</revision>
                  <publickey>ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQCiMX5AjFoK+B8bEts97OEKkJmONy8wDVSTe4Sx356p1fd48UQHq0g3xphfEWqZNVEvvXyVT3ToJZpsn6ZXALR6awp1EosV0Y3eCRn3HJ7VFifsObEBaJlbIpPWO3a44yQuNmB18dpAZsOYF0fuv9O9JZF/r2aS3DwJKKvrb1raPtuOkmLvMFOyFzQ4CzbpzOhxfLyk4VyyqWtxgRWa3cLJRC1s8pZP+Eeujz9lUXJOBJkz458myjcNogZ60HqMWPmNEQxKQKxKz5s1KhTzEa13AbK3mfBz6GYRSUE4PgzPNGt3ggKjm109MCECVLJ20i41l1x0LQogH4io0zN1KGFJ</publickey>
                  <signature>11958004301656338144383334445736600827209441963611815598420603918822211142526966075720891212808194909550288382243052852622156637099050410373511336910133303791008042473718309984723895304895629463705900053550743363551114431658248065643054664707971189331530190824121429353343469077985523096241437248019113197103461636847516945163199302563987603148811708018950051799390582546914509552932256650563055856190431745640579914653327517317125317899208768049182599520431852111466415308484761163553541804212696743391960660588998008501810208556514930557535867983962893715967502497080388797303899102334976976072133672653305360264021</signature>
                </identity>""",
        }
        some_identity = identity.identity(xmlsrc=known.get(idname))
        self.assertTrue(some_identity.isCorrect())
        self.assertTrue(some_identity.Valid())
        id_url.identity_cached(some_identity)
        return some_identity

    def test_exceptions(self):
        with self.assertRaises(KeyError):
            id_url.field(alice_bin).to_public_key()
        with self.assertRaises(KeyError):
            (id_url.field(alice_bin) in {id_url.field(alice_bin): 'abc', })
        with self.assertRaises(KeyError):
            (alice_bin in {id_url.field(alice_bin): 'abc', })
        with self.assertRaises(KeyError):
            (id_url.field(alice_bin) == id_url.field(alice_bin))
        with self.assertRaises(KeyError):
            (id_url.field(alice_bin) == id_url.field(bob))
        with self.assertRaises(TypeError):
            (alice_bin in [id_url.field(alice_bin), ])
        with self.assertRaises(TypeError):
            (id_url.field(alice_text) == alice_bin)
        with self.assertRaises(TypeError):
            (id_url.field(alice_bin) == alice_text)
        with self.assertRaises(TypeError):
            (id_url.field(alice_bin) in [alice_bin, ])
        with self.assertRaises(TypeError):
            (id_url.field(alice_bin) not in [alice_bin, ])

    def test_identity_cached(self):
        self.assertFalse(id_url.is_cached(alice_bin))
        self.assertFalse(id_url.is_cached(alice_text))
        alice_identity = self._cache_identity('alice')
        self.assertTrue(id_url.is_cached(alice_bin))
        self.assertTrue(id_url.is_cached(alice_text))
        self.assertEqual(alice_identity.getIDURL().to_bin(), alice_bin)
        self.assertEqual(alice_identity.getIDURL().to_text(), alice_text)
        self.assertEqual(id_url.field(alice_bin).to_public_key(), alice_identity.getPublicKey())
        self.assertEqual(id_url.field(alice_text), id_url.field(alice_bin))
        self._cache_identity('bob')
        self.assertTrue(id_url.field(alice_text) != id_url.field(bob))
 
    def test_identity_not_cached(self):
        self._cache_identity('alice')
        with self.assertRaises(KeyError):
            (id_url.field(ethan_text) != id_url.field(alice_bin))
        l = [id_url.field(frank_2), ]
        with self.assertRaises(KeyError):
            (id_url.field(frank_1) not in l)
        self._cache_identity('frank')
        self.assertIn(id_url.field(frank_1), l)
   
    def test_empty(self):
        self.assertTrue(id_url.is_empty(id_url.field(b'')))
        self.assertTrue(id_url.is_empty(id_url.field('')))
        self.assertTrue(id_url.is_empty(id_url.field(None)))
        self.assertTrue(id_url.is_empty(id_url.field(b'None')))
        self.assertTrue(id_url.is_empty(None))
        self.assertTrue(id_url.is_empty(b''))
        self.assertTrue(id_url.is_empty(''))
        self.assertTrue(id_url.is_empty(b'None'))
        self.assertTrue(id_url.is_empty('None'))
        self.assertFalse(id_url.field(b''))
        self.assertFalse(id_url.field(''))
        self.assertFalse(id_url.field(None))
        self.assertFalse(id_url.field('None'))
        self.assertFalse(id_url.field(b'None'))
        self.assertTrue(bool(id_url.field(b'')) is False)
        self.assertTrue(bool(id_url.field('')) is False)
        self.assertTrue(bool(id_url.field(None)) is False)
        self.assertTrue(bool(id_url.field('None')) is False)
        self.assertTrue(bool(id_url.field(b'None')) is False)
        self.assertTrue(id_url.field(b'') is not None)
        self.assertTrue(id_url.field('') is not None)
        self.assertFalse(id_url.field(None) is None)
        self.assertTrue(id_url.field(None) is not None)
        self.assertTrue(id_url.field(b'') == b'')
        self.assertTrue(id_url.field('') == '')
        self.assertTrue(id_url.field('') == b'')
        self.assertTrue(id_url.field(None) == '')
        self.assertTrue(id_url.field(None) == b'')
        l = [b'', None, '', id_url.field(''), id_url.field(None), id_url.field(b''), ]
        self.assertIn(b'', l)
        self.assertIn('', l)
        self.assertIn(None, l)
        self.assertTrue(id_url.is_some_empty(l))
        self.assertEqual(l.count(None), 4)
        self.assertEqual(id_url.empty_count(l), 6)
        self.assertTrue(None in [id_url.field(None), ])
        self.assertTrue(None in [id_url.field(b''), ])
        self.assertTrue(b'' in [id_url.field(None), ])
        self.assertTrue(b'' in [id_url.field(b''), ])
        self.assertTrue(id_url.is_in(None, [id_url.field(None), ]))
        self.assertTrue(id_url.is_in(None, [id_url.field(b''), ]))
        self.assertTrue(id_url.is_in(b'', [id_url.field(None), ]))
        self.assertTrue(id_url.is_in(b'', [id_url.field(b''), ]))
        d = {id_url.field(''): 0, id_url.field(None): 1, id_url.field(b''): 2}
        self.assertTrue(len(d), 1)
        self.assertTrue(b'' in d)
        self.assertTrue('' in d)
        self.assertFalse(b'' not in d)
        self.assertFalse('' not in d)
        self.assertNotIn(None, d)
        self.assertIn(id_url.field(''), d)
        self.assertIn(id_url.field(b''), d)
        self.assertIn(id_url.field(None), d)

    def test_not_empty(self):
        self.assertFalse(id_url.is_empty(id_url.field(ethan_bin)))
        self.assertFalse(id_url.is_empty(id_url.field(ethan_text)))
        self.assertTrue(id_url.field(ethan_bin))
        self.assertTrue(id_url.field(ethan_text))
        self.assertTrue(bool(id_url.field(ethan_text)) is True)
        self.assertTrue(id_url.field(ethan_text).to_bin() != dave)

    def test_in_list(self):
        self._cache_identity('alice')
        self._cache_identity('bob')
        self._cache_identity('carl')
        l = [
            id_url.field(alice_text),
            id_url.field(bob),
            id_url.field(frank_1),
            id_url.field(frank_2),
        ]
        with self.assertRaises(KeyError):
            (id_url.field(carl) not in l)
        self.assertTrue(id_url.field(alice_text) in l)
        with self.assertRaises(KeyError):
            (id_url.field(b'http://fake.com/frank.xml') not in l)
        with self.assertRaises(KeyError):
            (id_url.field(ethan_not_exist) not in l)
        self.assertTrue(id_url.field(bob) in l)
        with self.assertRaises(KeyError):
            (id_url.field(frank_1) in l)
        with self.assertRaises(KeyError):
            (id_url.field(frank_2) in l)
        self.assertTrue(len(l), 4)
        with self.assertRaises(KeyError):
            (l[0] != l[3])
        with self.assertRaises(KeyError):
            (l[2] == l[3])
        self._cache_identity('frank')
        self.assertTrue(l[2] == l[3])
        self.assertTrue(l[0] != l[3])
        self.assertIn(id_url.field(frank_1), l)
        self.assertFalse(id_url.is_some_empty(l))
        self.assertEqual(l.count(None), 0)
        self.assertEqual(id_url.empty_count(l), 0)
        l += [id_url.field(b''), ] * 3
        self.assertEqual(l.count(None), 3)
        self.assertEqual(l.count(b''), 3)
        self.assertEqual(l.count(''), 3)
        self.assertEqual(l.count(id_url.field(None)), 3)
        self.assertEqual(l.count(id_url.field(b'')), 3)
        self.assertEqual(l.count(id_url.field('')), 3)
        self.assertEqual(id_url.fields_list([b'', ]), [b'', ])
        self.assertEqual(id_url.fields_list([b'', ]), [None, ])
        self.assertEqual(id_url.fields_list([id_url.field(''), ]), [None, ])
        self.assertEqual(id_url.fields_list([None, ]), [id_url.field(''), ])
        self.assertNotEqual(id_url.fields_list([None, None]), [id_url.field(''), ])
        self.assertEqual(len(id_url.fields_list([None, ])), 1)

    def test_in_dict(self):
        self._cache_identity('alice')
        self._cache_identity('bob')
        self._cache_identity('carl')
        d = {}
        d[id_url.field(alice_text)] = 'alice'
        d[id_url.field(bob)] = 'bob'
        self.assertIn(id_url.field(alice_text), d)
        self.assertNotIn(id_url.field(carl), d)
        self.assertTrue(id_url.is_in(alice_text, d))
        self.assertTrue(id_url.is_not_in(carl, d))
        d2 = {id_url.field(bob): 'bob', 'some_key': 'some_value', }
        self.assertIn('some_key', d2)
        keys = list(d2.keys())
        with self.assertRaises(TypeError):
            (keys[0] != keys[1])

    def test_dict_of_lists(self):
        self._cache_identity('alice')
        self._cache_identity('bob')
        self._cache_identity('carl')
        self._cache_identity('frank')
        d = {}
        d[id_url.field(alice_text)] = []
        d[id_url.field(bob)] = []
        d[id_url.field(alice_text)].append(id_url.field(carl))
        d[id_url.field(alice_text)].append(id_url.field(frank_1))
        d[id_url.field(bob)].append(id_url.field(''))
        d[id_url.field(bob)].append(id_url.field(frank_2))
        d[id_url.field(bob)].append(id_url.field(None))
        d[id_url.field(bob)].append(id_url.field(b''))
        self.assertIn(id_url.field(frank_1), d[id_url.field(alice_text)])
        self.assertIn(id_url.field(frank_1), d[id_url.field(bob)])
        self.assertFalse(id_url.is_some_empty(d[id_url.field(alice_text)]))
        self.assertTrue(id_url.is_some_empty(d[id_url.field(bob)]))
        self.assertEqual(len(id_url.fields_list(d[id_url.field(bob)])), 4)

    def test_two_sources(self):
        with self.assertRaises(KeyError):
            {id_url.field(frank_1): 'frank1', }
        with self.assertRaises(KeyError):
            (id_url.field(frank_1) in {})
        with self.assertRaises(KeyError):
            (id_url.field(frank_2) in {})
        self._cache_identity('frank')
        d1 = {id_url.field(frank_1): 'frank1', }
        d2 = {id_url.field(frank_2): 'frank2', }
        self.assertEqual(id_url.field(frank_1), id_url.field(frank_2))
        self.assertIn(id_url.field(frank_1), d1)
        self.assertIn(id_url.field(frank_2), d2)
        self.assertIn(id_url.field(frank_1), d2)
        self.assertIn(id_url.field(frank_2), d1)
        self.assertTrue(id_url.is_in(frank_1, d2))
        self.assertTrue(id_url.is_in(frank_2, d2))
        self.assertEqual(list(d1.keys())[0], list(d2.keys())[0])
        self.assertEqual(list(d1.values())[0], 'frank1')
        self.assertEqual(d1.keys(), d2.keys())
        self.assertNotEqual(d1[id_url.field(frank_1)], d2[id_url.field(frank_2)])
        self.assertEqual(d1[id_url.field(frank_1)], d1[id_url.field(frank_2)])

    def test_three_sources(self):
        self._cache_identity('george')
        d3 = {
            id_url.field(george_1),
            id_url.field(george_2),
        }
        self.assertEqual(len(d3), 1)
        self.assertEqual(id_url.field(george_2), id_url.field(george_1))
        self.assertEqual(id_url.field(george_3), id_url.field(george_1))
        self.assertIn(id_url.field(george_1), d3)
        self.assertIn(id_url.field(george_2), d3)
        self.assertIn(id_url.field(george_3), d3)
        self._cache_identity('frank')
        self.assertNotIn(id_url.field(frank_1), d3)
        self.assertNotIn(id_url.field(frank_2), d3)
        self.assertFalse(id_url.is_in(frank_1, d3))
        self.assertTrue(id_url.is_in(george_1, d3))
        self.assertTrue(id_url.is_in(george_2, d3))
        self.assertTrue(id_url.is_in(george_3, d3))

    def test_fake_name(self):
        self._cache_identity('frank')
        self._cache_identity('fake_frank')
        self.assertNotEqual(id_url.field(fake_frank).original(), id_url.field(frank_1).original())
        self.assertNotEqual(id_url.field(fake_frank).original(), id_url.field(frank_2).original())
        self.assertNotEqual(id_url.field(fake_frank), id_url.field(frank_1))
        self.assertEqual(id_url.field(fake_frank), id_url.field(frank_2))
        self.assertNotEqual(id_url.field(fake_frank).to_public_key(), id_url.field(frank_1).to_public_key())
        self.assertEqual(id_url.field(fake_frank).to_public_key(), id_url.field(frank_2).to_public_key())

    def test_latest_vs_original(self):
        self._cache_identity('hans1')
        self._cache_identity('hans2')
        self._cache_identity('hans3')
        self.assertEqual(id_url.field(hans1), id_url.field(hans2))
        self.assertNotEqual(id_url.field(hans1).original(), id_url.field(hans2).original())
        self.assertEqual(id_url.field(hans1), id_url.field(hans3))
        self.assertNotEqual(id_url.field(hans1).original(), id_url.field(hans3).original())
        self.assertEqual(id_url.field(hans3), id_url.field(hans2))
        self.assertNotEqual(id_url.field(hans3).original(), id_url.field(hans2).original())
        self.assertEqual(id_url.field(hans1).to_text(), hans3)
        self.assertEqual(id_url.field(hans2).to_text(), hans3)
        self.assertEqual(id_url.field(hans3).to_text(), hans3)

    def test_latest_revision_order_123(self):
        self._cache_identity('hans1')
        self._cache_identity('hans2')
        self._cache_identity('hans3')
        self.assertNotEqual(id_url.field(hans1).to_text(), hans1)
        self.assertNotEqual(id_url.field(hans2).to_text(), hans1)
        self.assertNotEqual(id_url.field(hans3).to_text(), hans1)
        self.assertNotEqual(id_url.field(hans1).to_text(), hans2)
        self.assertNotEqual(id_url.field(hans2).to_text(), hans2)
        self.assertNotEqual(id_url.field(hans3).to_text(), hans2)
        self.assertEqual(id_url.field(hans1).to_text(), hans3)
        self.assertEqual(id_url.field(hans2).to_text(), hans3)
        self.assertEqual(id_url.field(hans3).to_text(), hans3)
        self.assertEqual(id_url.field(hans1).original(), strng.to_bin(hans1))
        self.assertEqual(id_url.field(hans2).original(), strng.to_bin(hans2))
        self.assertEqual(id_url.field(hans3).original(), strng.to_bin(hans3))

    def test_latest_revision_order_321(self):
        self._cache_identity('hans3')
        self._cache_identity('hans2')
        self._cache_identity('hans1')
        self.assertNotEqual(id_url.field(hans1).to_text(), hans1)
        self.assertNotEqual(id_url.field(hans2).to_text(), hans1)
        self.assertNotEqual(id_url.field(hans3).to_text(), hans1)
        self.assertNotEqual(id_url.field(hans1).to_text(), hans2)
        self.assertNotEqual(id_url.field(hans2).to_text(), hans2)
        self.assertNotEqual(id_url.field(hans3).to_text(), hans2)
        self.assertEqual(id_url.field(hans1).to_text(), hans3)
        self.assertEqual(id_url.field(hans2).to_text(), hans3)
        self.assertEqual(id_url.field(hans3).to_text(), hans3)
        self.assertEqual(id_url.field(hans1).original(), strng.to_bin(hans1))
        self.assertEqual(id_url.field(hans2).original(), strng.to_bin(hans2))
        self.assertEqual(id_url.field(hans3).original(), strng.to_bin(hans3))


if __name__ == "__main__":
    unittest.main()



from unittest import TestCase
import time

import subprocess


class TestMakeRead(TestCase):

    def setUp(self):
        t = time.time()

        self.dir_to_test = 'tests/data/test_%s' % t
        self.dir_to_files = 'tests/data/test_%s/bitdust' % t

        subprocess.call(['mkdir', self.dir_to_test])
        subprocess.call(['mkdir', self.dir_to_files])

        self.dir_to_restored_file = '%s/restored_file' % self.dir_to_files

    def tearDown(self):
        subprocess.call(['rm', '-rf', self.dir_to_test])

    def _test_file(self, filename):
        dir_to_origin_file = 'tests/data/%s' % filename

        subprocess.call(['cp', dir_to_origin_file, self.dir_to_files])

        dir_to_copied_file = '%s/%s' % (self.dir_to_files, filename)
        dir_to_test = self.dir_to_test
        dir_to_files = self.dir_to_files
        dir_to_restored_file = self.dir_to_restored_file

        subprocess.call(['python3', 'raid/make.py', dir_to_copied_file, 'ecc/4x4', 'myID_ABC', '100', dir_to_files])

        subprocess.call(['python3', 'raid/read.py', dir_to_restored_file, 'ecc/4x4', 'bitdust', '100', dir_to_test])
        r = subprocess.call(['diff', dir_to_restored_file, dir_to_copied_file])
        self.assertEqual(r, 0, 'ERROR %s' % r)

        subprocess.call(['rm', '%s/100-0-Data' % dir_to_files])

        subprocess.call(['python3', 'raid/read.py', dir_to_restored_file, 'ecc/4x4', 'bitdust', '100', dir_to_test])
        r = subprocess.call(['diff', dir_to_restored_file, dir_to_copied_file])
        self.assertEqual(r, 0, 'ERROR %s' % r)

        subprocess.call(['rm', '%s/100-1-Data' % dir_to_files])

        subprocess.call(['python3', 'raid/read.py', dir_to_restored_file, 'ecc/4x4', 'bitdust', '100', dir_to_test])
        r = subprocess.call(['diff', dir_to_restored_file, dir_to_copied_file])
        self.assertEqual(r, 0, 'ERROR %s' % r)

        subprocess.call(['rm', '%s/100-2-Data' % dir_to_files])

        subprocess.call(['python3', 'raid/read.py', dir_to_restored_file, 'ecc/4x4', 'bitdust', '100', dir_to_test])
        r = subprocess.call(['diff', dir_to_restored_file, dir_to_copied_file])
        self.assertEqual(r, 0, 'ERROR %s' % r)

        subprocess.call(['rm', '%s/100-3-Data' % dir_to_files])

        subprocess.call(['python3', 'raid/read.py', dir_to_restored_file, 'ecc/4x4', 'bitdust', '100', dir_to_test])
        r = subprocess.call(['diff', dir_to_restored_file, dir_to_copied_file])
        self.assertEqual(r, 0, 'ERROR %s' % r)

    def test_small_file(self):
        self._test_file('bitdust.png')



#!/usr/bin/python
# filemanager_api.py
#
# Copyright (C) 2008 Veselin Penev, https://bitdust.io
#
# This file (filemanager_api.py) is part of BitDust Software.
#
# BitDust is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# BitDust Software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with BitDust Software.  If not, see <http://www.gnu.org/licenses/>.
#
# Please contact us if you have any questions at bitdust.io@gmail.com
#
#
#
#

"""
..

module:: filemanager_api
"""

#------------------------------------------------------------------------------

from __future__ import absolute_import

#------------------------------------------------------------------------------

import os
import sys
import time
import pprint

#------------------------------------------------------------------------------

from lib import strng

from logs import lg

from system import bpio

from lib import packetid
from lib import misc

from main import settings
from main import control

from services import driver

from transport import packet_in
from transport import packet_out

from storage import backup_fs
from storage import backup_control
from storage import backup_monitor
from storage import restore_monitor

from userid import my_id
from userid import global_id

#------------------------------------------------------------------------------


def process(json_request):
    lg.out(12, 'filemanager_api.process %s' % json_request)
    if not driver.is_on('service_my_data'):
        return {'result': {
            "success": False,
            "error": "network [service_my_data] is not started: %s" % (
                driver.services().get('service_my_data', '!!! not found !!!'))}}
    mode = ''
    result = {}
    try:
        if strng.is_string(json_request):
            import json
            json_request = json.loads(json_request)
        mode = json_request['params']['mode']
        if mode == 'config':
            result = _config(json_request['params'])
        elif mode == 'stats':
            result = _stats(json_request['params'])
        elif mode == 'list':
            result = _list(json_request['params'])
        elif mode == 'listlocal':
            result = _list_local(json_request['params'])
        elif mode == 'listall':
            result = _list_all(json_request['params'])
        elif mode == 'upload':
            result = _upload(json_request['params'])
        elif mode == 'delete':
            result = _delete(json_request['params'])
        elif mode == 'deleteversion':
            result = _delete_version(json_request['params'])
        elif mode == 'download':
            result = _download(json_request['params'])
        elif mode == 'tasks':
            result = _list_active_tasks(json_request['params'])
        elif mode == 'packets':
            result = _list_in_out_packets(json_request['params'])
        elif mode == 'connections':
            result = _list_active_connections(json_request['params'])
        elif mode == 'streams':
            result = _list_active_streams(json_request['params'])
        elif mode == 'debuginfo':
            result = _debuginfo(json_request['params'])
        else:
            result = {"result": {"success": False,
                                 "error": 'filemanager method %s not found' % mode}}
    except Exception as exc:
        lg.exc()
        descr = str(sys.exc_info()[0].__name__) + ': ' + str(sys.exc_info()[1])
        result = {"result": {"success": False,
                             "error": descr}}
    # lg.out(4, '    ERROR unknown mode: %s' % mode)
    lg.out(20, '    %s' % pprint.pformat(result))
    return result

#------------------------------------------------------------------------------


def _config(params):
    result = []
    homepath = bpio.portablePath(os.path.expanduser('~'))
    if bpio.Windows():
        # set "c:" as a starting point when pick files for Windows
        # probably should be MyDocuments folder or something else,
        # but lets take that for now
        homepath = homepath[:2]
    result.append({'key': 'homepath',
                   'value': homepath})
    return {'result': result, }


def _stats(params):
    from contacts import contactsdb
    from p2p import online_status
    from lib import diskspace
    result = {}
    result['suppliers'] = contactsdb.num_suppliers()
    result['max_suppliers'] = settings.getSuppliersNumberDesired()
    result['online_suppliers'] = online_status.countOnlineAmong(contactsdb.suppliers())
    result['customers'] = contactsdb.num_customers()
    result['bytes_donated'] = settings.getDonatedBytes()
    result['value_donated'] = diskspace.MakeStringFromBytes(settings.getDonatedBytes())
    result['bytes_needed'] = settings.getNeededBytes()
    result['value_needed'] = diskspace.MakeStringFromBytes(settings.getNeededBytes())
    result['bytes_used_total'] = backup_fs.sizebackups()
    result['value_used_total'] = diskspace.MakeStringFromBytes(backup_fs.sizebackups())
    result['bytes_used_supplier'] = 0 if (contactsdb.num_suppliers() == 0) else (int(backup_fs.sizebackups() / contactsdb.num_suppliers()))
    result['bytes_indexed'] = backup_fs.sizefiles() + backup_fs.sizefolders()
    result['files_count'] = backup_fs.numberfiles()
    result['folders_count'] = backup_fs.numberfolders()
    result['items_count'] = backup_fs.counter()
    result['timestamp'] = time.time()
    return {'result': result, }


def _list(params):
    result = []
    path = params['path']
    if bpio.Linux() or bpio.Mac():
        path = '/' + path.lstrip('/')
    lst = backup_fs.ListByPathAdvanced(path)
    if not isinstance(lst, list):
        lg.warn('backup_fs.ListByPathAdvanced returned: %s' % lst)
        return {"result": [], }
    for item in lst:
        if item[2] == 'index':
            continue
        result.append({
            "type": item[0],
            "name": item[1],
            "id": item[2],
            "rights": "",
            "size": item[3],
            "source_size": item[7].size,
            "date": item[4],
            "dirpath": item[5],
            "has_childs": item[6],
            "content": '1' if item[7].exist() else '',
            "versions": item[8],
        })
    return {'result': result, }


def _list_all(params):
    result = []
    lst = backup_fs.ListAllBackupIDsAdvanced()
    for item in lst:
        if item[2] == 'index':
            continue
        result.append({
            "type": item[0],
            "name": item[1],
            "id": item[2],
            "rights": "",
            "size": item[3],
            "date": item[4],
            "dirpath": item[5],
            "has_childs": item[6],
            "content": '1' if item[7] else '',
            "versions": item[8],
        })
    return {'result': result, }


def _list_local(params):
    result = []
    path = params['path']
    if bpio.Linux() or bpio.Mac():
        path = '/' + path.lstrip('/')
    path = bpio.portablePath(path)
    only_folders = params['onlyFolders']
    if (path == '' or path == '/') and bpio.Windows():
        for itemname in bpio.listLocalDrivesWindows():
            result.append({
                "name": itemname.rstrip('\\').rstrip('/').lower(),
                "rights": "drwxr-xr-x",
                "size": "",
                "date": "",
                "type": "dir",
                "dirpath": path,
            })
    else:
        if bpio.Windows() and len(path) == 2 and path[1] == ':':
            path += '/'
        apath = path
        for itemname in bpio.list_dir_safe(apath):
            itempath = os.path.join(apath, itemname)
            if only_folders and not os.path.isdir(itempath):
                continue
            result.append({
                "name": itemname,
                "rights": "drwxr-xr-x",
                "size": str(os.path.getsize(itempath)),
                "date": str(os.path.getmtime(itempath)),
                "type": "dir" if os.path.isdir(itempath) else "file",
                "dirpath": apath,
            })
    return {'result': result, }


def _upload(params):
    path = params['path']
    if bpio.Linux() or bpio.Mac():
        path = '/' + (path.lstrip('/'))
    localPath = strng.to_text(path)
    if not bpio.pathExist(localPath):
        return {'result': {"success": False, "error": 'local path %s was not found' % path}}
    result = []
    pathID = backup_fs.ToID(localPath)
    if pathID is None:
        if bpio.pathIsDir(localPath):
            pathID, iter, iterID = backup_fs.AddDir(localPath, read_stats=True)
            result.append('new folder was added: %s' % localPath)
        else:
            pathID, iter, iterID = backup_fs.AddFile(localPath, read_stats=True)
            result.append('new file was added: %s' % localPath)
    pathID = global_id.CanonicalID(pathID)
    backup_control.StartSingle(pathID=pathID, localPath=localPath)
    backup_fs.Calculate()
    backup_control.Save()
    control.request_update([('pathID', pathID), ])
    result.append('backup started: %s' % pathID)
    return {'result': result, }


def _download(params):
    # localName = params['name']
    backupID = global_id.CanonicalID(params['backupid'])
    destpath = params['dest_path']
    if bpio.Linux() or bpio.Mac():
        destpath = '/' + destpath.lstrip('/')
    restorePath = bpio.portablePath(destpath)
    # overwrite = params['overwrite']
    customerGlobalID, remotePath, version = packetid.SplitBackupID(backupID)
    pathID = packetid.MakeBackupID(customerGlobalID, remotePath)
    if not customerGlobalID:
        customerGlobalID = my_id.getGlobalID()
    if not packetid.IsCanonicalVersion(version):
        return {'result': {"success": False, "error": "path %s is not valid" % backupID}}
    if not remotePath:
        return {'result': {"success": False, "error": "path %s is not valid" % backupID}}
    if not packetid.Valid(remotePath):
        return {'result': {"success": False, "error": "path %s is not valid" % backupID}}
    if backup_control.IsBackupInProcess(backupID):
        return {'result': {"success": True, "error": None}}
    if backup_control.HasTask(pathID):
        return {'result': {"success": True, "error": None}}
    localPath = backup_fs.ToPath(remotePath)
    if localPath == restorePath:
        restorePath = os.path.dirname(restorePath)

    def _itemRestored(backupID, result):
        customerGlobalID, remotePath, _ = packetid.SplitBackupID(backupID)
        backup_fs.ScanID(remotePath, customer_idurl=global_id.GlobalUserToIDURL(customerGlobalID))
        backup_fs.Calculate()

    restore_monitor.Start(backupID, restorePath, callback=_itemRestored)
    return {'result': {"success": True, "error": None}}


def _delete(params):
    # localPath = params['path'].lstrip('/')
    pathID = params['id']
    if not packetid.Valid(pathID):
        return {'result': {"success": False, "error": "path %s is not valid" % pathID}}
    if not backup_fs.ExistsID(pathID):
        return {'result': {"success": False, "error": "path %s not found" % pathID}}
    backup_control.DeletePathBackups(pathID, saveDB=False, calculate=False)
    backup_fs.DeleteLocalDir(settings.getLocalBackupsDir(), pathID)
    backup_fs.DeleteByID(pathID)
    backup_fs.Scan()
    backup_fs.Calculate()
    backup_control.Save()
    control.request_update([('pathID', pathID), ])
    backup_monitor.A('restart')
    return {'result': {"success": True, "error": None}}


def _delete_version(params):
    lg.out(6, '_delete_version %s' % str(params))
    backupID = params['backupid']
    if not packetid.Valid(backupID):
        return {'result': {"success": False, "error": "backupID %s is not valid" % backupID}}
    customerGlobalID, remotePath, version = packetid.SplitBackupID(backupID)
    if not customerGlobalID:
        customerGlobalID = my_id.getGlobalID()
    if not backup_fs.ExistsID(remotePath, iterID=backup_fs.fsID(global_id.GlobalUserToIDURL(customerGlobalID))):
        return {'result': {"success": False, "error": "path %s not found" % remotePath}}
    if version:
        backup_control.DeleteBackup(backupID, saveDB=False, calculate=False)
    backup_fs.Scan()
    backup_fs.Calculate()
    backup_control.Save()
    backup_monitor.A('restart')
    control.request_update([('backupID', backupID), ])
    return {'result': {"success": True, "error": None}}


def _rename(params):
    return {'result': {"success": False, "error": "not done yet"}}


def _list_active_tasks(params):
    result = []
    for tsk in backup_control.ListPendingTasks():
        result.append({
            'name': os.path.basename(tsk.localPath),
            'path': os.path.dirname(tsk.localPath),
            'id': tsk.pathID,
            'version': '',
            'customer': '',
            'mode': 'up',
            'progress': '0%', })
    for backupID in backup_control.ListRunningBackups():
        backup_obj = backup_control.GetRunningBackupObject(backupID)
        customerGlobalID, remotePath, versionName = packetid.SplitBackupID(backupID)
        result.append({
            'name': os.path.basename(backup_obj.sourcePath),
            'path': os.path.dirname(backup_obj.sourcePath),
            'id': remotePath,
            'version': versionName,
            'customer': customerGlobalID,
            'mode': 'up',
            'progress': misc.percent2string(backup_obj.progress()), })
    # for backupID in restore_monitor.GetWorkingIDs():
    #     result.append(backupID)
    return {'result': result, }


def _list_in_out_packets(params):
    result = []
    for pkt_out in packet_out.queue():
        result.append({
            'name': pkt_out.outpacket.Command,
            'label': pkt_out.label,
            'from_to': 'to',
            'target': pkt_out.remote_idurl,
        })
    for pkt_in in list(packet_in.inbox_items()).values():
        result.append({
            'name': pkt_in.transfer_id,
            'label': pkt_in.label,
            'from_to': 'from',
            'target': pkt_in.sender_idurl,
        })
    return {'result': result, }


def _list_active_streams(params):
    result = []
    if not driver.is_on('service_gateway'):
        return {'result': result, }
    from transport import gateway
    result = []
    wanted_protos = params.get('protos', [])
    if not wanted_protos:
        wanted_protos = gateway.list_active_transports()
    for proto in wanted_protos:
        for stream in gateway.list_active_streams(proto):
            item = {
                'proto': proto,
                'id': '',
                'type': '',
                'bytes_current': -1,
                'bytes_total': -1,
                'progress': '0%',
            }
            if proto == 'tcp':
                if hasattr(stream, 'bytes_received'):
                    item.update({
                        'id': stream.file_id,
                        'type': 'in',
                        'bytes_current': stream.bytes_received,
                        'bytes_total': stream.size,
                        'progress': misc.value2percent(stream.bytes_received, stream.size, 0)
                    })
                elif hasattr(stream, 'bytes_sent'):
                    item.update({
                        'id': stream.file_id,
                        'type': 'out',
                        'bytes_current': stream.bytes_sent,
                        'bytes_total': stream.size,
                        'progress': misc.value2percent(stream.bytes_sent, stream.size, 0)
                    })
            elif proto == 'udp':
                if hasattr(stream.consumer, 'bytes_received'):
                    item.update({
                        'id': stream.stream_id,
                        'type': 'in',
                        'bytes_current': stream.consumer.bytes_received,
                        'bytes_total': stream.consumer.size,
                        'progress': misc.value2percent(stream.consumer.bytes_received, stream.consumer.size, 0)
                    })
                elif hasattr(stream.consumer, 'bytes_sent'):
                    item.update({
                        'id': stream.stream_id,
                        'type': 'out',
                        'bytes_current': stream.consumer.bytes_sent,
                        'bytes_total': stream.consumer.size,
                        'progress': misc.value2percent(stream.consumer.bytes_sent, stream.consumer.size, 0)
                    })
            result.append(item)
    return {'result': result, }


def _list_active_connections(params):
    result = []
    if not driver.is_on('service_gateway'):
        return {'result': result, }
    from transport import gateway
    result = []
    wanted_protos = params.get('protos', [])
    if not wanted_protos:
        wanted_protos = gateway.list_active_transports()
    for proto in wanted_protos:
        for connection in gateway.list_active_sessions(proto):
            item = {
                'status': 'unknown',
                'state': 'unknown',
                'proto': proto,
                'host': 'unknown',
                'idurl': 'unknown',
                'bytes_sent': 0,
                'bytes_received': 0,
            }
            if proto == 'tcp':
                if hasattr(connection, 'stream'):
                    try:
                        host = '%s:%s' % (connection.peer_address[0], connection.peer_address[1])
                    except:
                        host = 'unknown'
                    item.update({
                        'status': 'active',
                        'state': connection.state,
                        'host': host,
                        'idurl': connection.peer_idurl or '',
                        'bytes_sent': connection.total_bytes_sent,
                        'bytes_received': connection.total_bytes_received,
                    })
                else:
                    try:
                        host = '%s:%s' % (connection.connection_address[0], connection.connection_address[1])
                    except:
                        host = 'unknown'
                    item.update({
                        'status': 'connecting',
                        'host': host,
                    })
            elif proto == 'udp':
                try:
                    host = '%s:%s' % (connection.peer_address[0], connection.peer_address[1])
                except:
                    host = 'unknown'
                item.update({
                    'status': 'active',
                    'state': connection.state,
                    'host': host,
                    'idurl': connection.peer_idurl or '',
                    'bytes_sent': connection.bytes_sent,
                    'bytes_received': connection.bytes_received,
                })
            result.append(item)
    return {'result': result, }


def _debuginfo(params):
    result = {}
    result['debug'] = lg.get_debug_level()
    result['automats'] = []
    from automats import automat
    for index, A in automat.objects().items():
        result['automats'].append({
            'index': index,
            'id': A.id,
            'name': A.name,
            'state': A.state, })
    return {'result': result, }



from unittest import TestCase
import time
import mock

from raid.worker import Manager


def heavy_task(id):
    time.sleep(2)
    return id


def light_task():
    return None


# class TestManager(TestCase):
# 
#     def setUp(self):
#         self.manager = Manager(2)
# 
#     def tearDown(self):
#         self.manager.terminate()
# 
#     def test_submit(self):
#         callback = mock.Mock()
# 
#         self.manager.submit(func=light_task, params=(), callback=callback)
#         self.manager.submit(func=light_task, params=(), callback=callback)
#         self.manager.submit(func=light_task, params=(), callback=callback)
#         self.manager.submit(func=light_task, params=(), callback=callback)
#         self.manager.submit(func=light_task, params=(), callback=callback)
#         self.manager.submit(func=light_task, params=(), callback=callback)
#         self.manager.submit(func=light_task, params=(), callback=callback)
#         self.manager.submit(func=light_task, params=(), callback=callback)
#         self.manager.submit(func=light_task, params=(), callback=callback)
# 
#         time.sleep(5)
#         assert callback.call_count == 9, callback.call_count
# 
#     def test_callback(self):
#         heavy_task_1_callback = mock.Mock()
#         self.manager.submit(func=heavy_task, params=(1,), callback=heavy_task_1_callback)
#         time.sleep(5)
#         heavy_task_1_callback.assert_called_once_with(1)
# 
#     def test_cancellation(self):
#         heavy_task_1_callback = mock.Mock()
#         heavy_task_2_callback = mock.Mock()
#         heavy_task_3_callback = mock.Mock()
#         self.manager.submit(func=heavy_task, params=(1,), callback=heavy_task_1_callback)
#         self.manager.submit(func=heavy_task, params=(2,), callback=heavy_task_2_callback)
#         r3 = self.manager.submit(func=heavy_task, params=(3,), callback=heavy_task_3_callback)
# 
#         self.manager.cancel(r3.tid)
#         time.sleep(5)
# 
#         heavy_task_1_callback.assert_called_once()
#         heavy_task_2_callback.assert_called_once()
#         # heavy_task_3_callback.assert_not_called()



#!/usr/bin/env python
# utime.py
#
# Copyright (C) 2008 Veselin Penev, https://bitdust.io
#
# This file (utime.py) is part of BitDust Software.
#
# BitDust is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# BitDust Software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with BitDust Software.  If not, see <http://www.gnu.org/licenses/>.
#
# Please contact us if you have any questions at bitdust.io@gmail.com

"""
..

module:: utime
"""


from __future__ import absolute_import
import time
import datetime
import calendar


def utc_timestamp(d):
    timestamp1 = calendar.timegm(d.timetuple())
    return datetime.datetime.utcfromtimestamp(timestamp1)


def local_timestamp(d):
    timestamp2 = time.mktime(d.timetuple())
    return datetime.datetime.fromtimestamp(timestamp2)


def datetime_to_sec1970(dt):
    """
    Converts datetime object to seconds since 1970 year.
    """
    return int(time.mktime(dt.timetuple()))


def sec1970_to_datetime_utc(seconds=-1):
    """
    Converts seconds since 1970 year to datetime object in UTC form.
    """
    if seconds == -1:
        seconds = utcnow_to_sec1970()
    return datetime.datetime.utcfromtimestamp(seconds)


def utcnow_to_sec1970():
    """
    Returns how much seconds passed since 1970 till current moment depend on
    UTC timezone.
    """
    return datetime_to_sec1970(datetime.datetime.utcnow())


def get_sec1970():
    """
    Return how much seconds passed since 1970 using time.time() method, seems
    work in local time.
    TODO: extra methods for time synchronization across the network nodes to be added later
    """
    return int(time.time())


def make_timestamp():
    """
    Returns text string based on current time.
    """
    time_st = time.localtime()
    ampm = time.strftime("%p", time_st)
    if not ampm:
        ampm = 'AM' if time.time() % 86400 < 43200 else 'PM'
    return time.strftime("%Y%m%d%I%M%S", time_st) + ampm



#!/usr/bin/env python
# archive_writer.py
#
# Copyright (C) 2008 Veselin Penev, http://bitdust.io
#
# This file (archive_writer.py) is part of BitDust Software.
#
# BitDust is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# BitDust Software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with BitDust Software.  If not, see <http://www.gnu.org/licenses/>.
#
# Please contact us if you have any questions at bitdust.io@gmail.com


"""
.. module:: archive_writer
.. role:: red

BitDust archive_writer() Automat

EVENTS:
    * :red:`ack`
    * :red:`backup-done`
    * :red:`backup-failed`
    * :red:`block-ready`
    * :red:`dht-read-failed`
    * :red:`dht-read-success`
    * :red:`fail`
    * :red:`packets-delivered`
    * :red:`sending-failed`
    * :red:`start`
"""

#------------------------------------------------------------------------------

from __future__ import absolute_import

#------------------------------------------------------------------------------

_Debug = False
_DebugLevel = 10

#------------------------------------------------------------------------------

import os

#------------------------------------------------------------------------------

from logs import lg

from automats import automat

from system import bpio

from lib import packetid
from lib import misc

from main import settings

from crypt import my_keys

from dht import dht_relations

from p2p import commands
from p2p import p2p_service

from raid import eccmap

from storage import backup_fs
from storage import backup_tar
from storage import backup

from userid import global_id
from userid import my_id

#------------------------------------------------------------------------------

class ArchiveWriter(automat.Automat):
    """
    This class implements all the functionality of ``archive_writer()`` state machine.
    """

    def __init__(self, local_data_callback, debug_level=_DebugLevel, log_events=_Debug, log_transitions=_Debug, publish_events=False, **kwargs):
        """
        Builds `archive_writer()` state machine.
        """
        self.local_data_callback = local_data_callback
        super(ArchiveWriter, self).__init__(
            name="archive_writer",
            state="AT_STARTUP",
            debug_level=debug_level,
            log_events=log_events,
            log_transitions=log_transitions,
            publish_events=publish_events,
            **kwargs
        )

    def A(self, event, *args, **kwargs):
        """
        The state machine code, generated using `visio2python <http://bitdust.io/visio2python/>`_ tool.
        """
        #---AT_STARTUP---
        if self.state == 'AT_STARTUP':
            if event == 'start':
                self.state = 'DHT_READ?'
                self.doInit(*args, **kwargs)
                self.doDHTReadSuppliers(*args, **kwargs)
        #---DHT_READ?---
        elif self.state == 'DHT_READ?':
            if event == 'dht-read-success':
                self.state = 'BACKUP'
                self.doStartArchiveBackup(*args, **kwargs)
            elif event == 'dht-read-failed':
                self.state = 'FAILED'
                self.doReportFailed(event, *args, **kwargs)
                self.doDestroyMe(*args, **kwargs)
        #---BACKUP---
        elif self.state == 'BACKUP':
            if event == 'backup-done':
                self.state = 'SENDING'
                self.doCheckFinished(*args, **kwargs)
            elif event == 'block-ready':
                self.doPushPackets(*args, **kwargs)
            elif event == 'ack' or event == 'fail':
                self.doPullPacket(event, *args, **kwargs)
            elif event == 'backup-failed':
                self.state = 'FAILED'
                self.doReportFailed(*args, **kwargs)
                self.doDestroyMe(*args, **kwargs)
        #---SENDING---
        elif self.state == 'SENDING':
            if event == 'packets-delivered':
                self.state = 'DONE'
                self.doReportDone(*args, **kwargs)
                self.doDestroyMe(*args, **kwargs)
            elif event == 'ack' or event == 'fail':
                self.doPullPacket(event, *args, **kwargs)
                self.doCheckFinished(*args, **kwargs)
            elif event == 'sending-failed':
                self.state = 'FAILED'
                self.doReportFailed(*args, **kwargs)
                self.doDestroyMe(*args, **kwargs)
        #---DONE---
        elif self.state == 'DONE':
            pass
        #---FAILED---
        elif self.state == 'FAILED':
            pass
        return None

    def doInit(self, *args, **kwargs):
        """
        Action method.
        """
        self.queue_id = kwargs['queue_id']
        self.archive_info = kwargs['archive_info']
        self.archive_folder_path = kwargs['archive_folder_path']
        self.result_defer = kwargs.get('result_defer')
        qa, oid, _ = global_id.SplitGlobalQueueID(self.queue_id)
        self.queue_alias = qa
        self.queue_owner_id = oid
        self.queue_owner_idurl = global_id.glob2idurl(self.queue_owner_id)
        self.group_key_id = my_keys.make_key_id(alias=self.queue_alias, creator_glob_id=self.queue_owner_id)
        self.backup_job = None
        self.backup_max_block_num = None
        self.suppliers_list = []
        self.ecc_map = None
        self.correctable_errors = 0
        self.packets_out = {}

    def doDHTReadSuppliers(self, *args, **kwargs):
        """
        Action method.
        """
        d = dht_relations.read_customer_suppliers(customer_idurl=self.queue_owner_idurl, use_cache=True)
        d.addCallback(self._on_read_queue_owner_suppliers_success)
        d.addErrback(self._on_read_queue_owner_suppliers_failed)

    def doStartArchiveBackup(self, *args, **kwargs):
        """
        Action method.
        """
        self._do_start_archive_backup()

    def doPushPackets(self, *args, **kwargs):
        """
        Action method.
        """
        self._do_send_packets(
            backup_id=kwargs['backup_id'],
            block_num=kwargs['block_num'],
        )

    def doPullPacket(self, event, *args, **kwargs):
        """
        Action method.
        """
        newpacket = kwargs['newpacket']
        _, _, _, block_num, _, _ = packetid.SplitFull(newpacket.PacketID)
        if block_num not in self.packets_out:
            raise Exception('unregistered block number received')
        if newpacket.PacketID not in self.packets_out[block_num]:
            raise Exception('unregistered packet ID received')
        if event == 'ack':
            self.packets_out[block_num][newpacket.PacketID] = True
        else:
            self.packets_out[block_num][newpacket.PacketID] = False
        if _Debug:
            lg.args(_DebugLevel, event=event, block_num=block_num, packet_id=newpacket.PacketID, packets_out=self.packets_out)

    def doCheckFinished(self, *args, **kwargs):
        """
        Action method.
        """
        if _Debug:
            lg.args(_DebugLevel, backup_job=self.backup_job, backup_max_block_num=self.backup_max_block_num,
                    packets_out=list(self.packets_out.values()))
        if self.backup_job:
            # backup is not finished yet
            return
        if self.backup_max_block_num not in self.packets_out:
            # packets of the last block not sent yet
            return
        packets_in_progress = 0
        for block_num in self.packets_out.keys():
            packets_in_progress += list(self.packets_out[block_num].values()).count(None)
        if packets_in_progress:
            # some packets are still in progress
            return
        for block_num in self.packets_out.keys():
            block_packets_failed = list(self.packets_out[block_num].values()).count(False)
            if block_packets_failed > self.correctable_errors * 2:  # because each packet also have Parity()
                lg.err('all packets for block %d are sent, but too many errors: %d' % (block_num, block_packets_failed, ))
                self.automat('sending-failed')
                return
        self.automat('packets-delivered')

    def doReportDone(self, *args, **kwargs):
        """
        Action method.
        """
        if self.result_defer:
            self.result_defer.callback(self.archive_info)

    def doReportFailed(self, event, *args, **kwargs):
        """
        Action method.
        """
        ret = 'unknown error'
        if event == 'dht-read-failed':
            ret = 'failed reading list of suppliers from DHT'
        elif event == 'backup-failed':
            ret = 'local archive backup is failed'
        elif event == 'sending-failed':
            ret = 'sending archived packets failed'
        if self.result_defer:
            self.result_defer.errback(Exception(ret))

    def doDestroyMe(self, *args, **kwargs):
        """
        Remove all references to the state machine object to destroy it.
        """
        self.queue_id = None
        self.archive_info = None
        self.result_defer = None
        self.suppliers_list = None
        self.ecc_map = None
        self.packets_out = None
        self.block_failed = None
        self.backup_job = None
        self.correctable_errors = None
        self.destroy()

    def _do_start_archive_backup(self):
        archive_id, local_path = self.local_data_callback(self.queue_id, self.archive_info)
        supplier_path_id = os.path.join(self.archive_folder_path, archive_id)
        dataID = misc.NewBackupID()
        backup_id = packetid.MakeBackupID(
            customer=self.queue_owner_id,
            path_id=supplier_path_id,
            key_alias=self.queue_alias,
            version=dataID,
        )
        backup_fs.MakeLocalDir(settings.getLocalBackupsDir(), backup_id)
        if bpio.Android():
            compress_mode = 'none'
        else:
            compress_mode = 'bz2'
        arcname = os.path.basename(local_path)
        backupPipe = backup_tar.backuptarfile_thread(local_path, arcname=arcname, compress=compress_mode)
        self.backup_job = backup.backup(
            backupID=backup_id,
            pipe=backupPipe,
            blockResultCallback=self._on_archive_backup_block_result,
            finishCallback=self._on_archive_backup_done,
            blockSize=1024*1024*10,
            sourcePath=local_path,
            keyID=self.group_key_id,
            ecc_map=eccmap.eccmap(self.ecc_map),
            creatorIDURL=self.queue_owner_idurl,
        )
        self.backup_job.automat('start')
        if _Debug:
            lg.args(_DebugLevel, job=self.backup_job, backup_id=backup_id, local_path=local_path, group_key_id=self.group_key_id)

    def _do_send_packets(self, backup_id, block_num):
        customer_id, path_id, version_name = packetid.SplitBackupID(backup_id)
        archive_snapshot_dir = os.path.join(settings.getLocalBackupsDir(), customer_id, path_id, version_name)
        if _Debug:
            lg.args(_DebugLevel, backup_id=backup_id, block_num=block_num, archive_snapshot_dir=archive_snapshot_dir)
        if not os.path.isdir(archive_snapshot_dir):
            self.block_failed = True
            lg.err('archive snapshot folder was not found in %r' % archive_snapshot_dir)
            return None
        failed_supliers = 0
        for supplier_num in range(len(self.suppliers_list)):
            supplier_idurl = self.suppliers_list[supplier_num]
            if not supplier_idurl:
                failed_supliers += 1
                lg.warn('unknown supplier supplier_num=%d' % supplier_num)
                continue
            for dataORparity in ('Data', 'Parity', ):
                packet_id = packetid.MakePacketID(backup_id, block_num, supplier_num, dataORparity)
                packet_filename = os.path.join(archive_snapshot_dir, '%d-%d-%s' % (
                    block_num, supplier_num, dataORparity,
                ))
                if not os.path.isfile(packet_filename):
                    lg.err('%s is not a file' % packet_filename)
                    continue
                packet_payload = bpio.ReadBinaryFile(packet_filename)
                if not packet_payload:
                    lg.err('file %r reading error' % packet_filename)
                    continue
                if block_num not in self.packets_out:
                    self.packets_out[block_num] = {}
                self.packets_out[block_num][packet_id] = None
                p2p_service.SendData(
                    raw_data=packet_payload,
                    ownerID=self.queue_owner_idurl,
                    creatorID=my_id.getIDURL(),
                    remoteID=supplier_idurl,
                    packetID=packet_id,
                    callbacks={
                        commands.Ack(): lambda newpacket, _: self.automat('ack', newpacket=newpacket),
                        commands.Fail(): lambda newpacket, _: self.automat('fail', newpacket=newpacket),
                    },
                )
        if failed_supliers > self.correctable_errors:
            self.block_failed = True
            lg.err('too many failed suppliers %d in block %d' % (failed_supliers, block_num, ))

    def _on_read_queue_owner_suppliers_success(self, dht_value):
        # TODO: add more validations of dht_value
        if dht_value and isinstance(dht_value, dict) and len(dht_value.get('suppliers', [])) > 0:
            self.suppliers_list = dht_value['suppliers']
            self.ecc_map = dht_value['ecc_map']
            self.correctable_errors = eccmap.GetCorrectableErrors(len(self.suppliers_list))
        if _Debug:
            lg.args(_DebugLevel, suppliers_list=self.suppliers_list, ecc_map=self.ecc_map)
        if not self.suppliers_list or not self.ecc_map:
            self.automat('dht-read-failed', None)
            return None
        self.automat('dht-read-success')
        return None

    def _on_read_queue_owner_suppliers_failed(self, err):
        lg.err('failed to read customer suppliers: %r' % err)
        self.automat('dht-read-failed', err)
        return None

    def _on_archive_backup_block_result(self, backup_id, block_num, result):
        if not result:
            self.block_failed = True
            return
        self.automat('block-ready', backup_id=backup_id, block_num=block_num)

    def _on_archive_backup_done(self, backup_id, result):
        self.backup_max_block_num = self.backup_job.blockNumber
        self.backup_job = None
        if result != 'done':
            self.automat('backup-failed')
            return
        self.automat('backup-done')



#!/usr/bin/python
# service_http_transport.py
#
# Copyright (C) 2008 Veselin Penev, https://bitdust.io
#
# This file (service_http_transport.py) is part of BitDust Software.
#
# BitDust is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# BitDust Software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with BitDust Software.  If not, see <http://www.gnu.org/licenses/>.
#
# Please contact us if you have any questions at bitdust.io@gmail.com
#
#
#
#

"""
..

module:: service_http_transport
"""

from __future__ import absolute_import
from services.local_service import LocalService


def create_service():
    return HTTPTransportService()


class HTTPTransportService(LocalService):

    service_name = 'service_http_transport'
    config_path = 'services/http-transport/enabled'
    proto = 'http'

    def enabled(self):
        # TODO: development just started... service disabled at the moment
        return False

    def dependent_on(self):
        return [
            'service_http_connections',
            'service_gateway',
        ]

    def installed(self):
        # TODO: to be continue...
        return False

    def start(self):
        from twisted.internet import reactor  # @UnresolvedImport
        from twisted.internet.defer import Deferred
        from transport.http import http_interface
        from transport import network_transport
        from transport import gateway
        from main.config import conf
        self.starting_deferred = Deferred()
        self.transport = network_transport.NetworkTransport('http', http_interface.GateInterface())
        self.transport.automat('init',
                               (gateway.listener(), self._on_transport_state_changed))
        reactor.callLater(0, self.transport.automat, 'start')  # @UndefinedVariable
        conf().addConfigNotifier('services/http-transport/enabled',
                           self._on_enabled_disabled)
        conf().addConfigNotifier('services/http-transport/receiving-enabled',
                           self._on_receiving_enabled_disabled)
        return self.starting_deferred

    def stop(self):
        from main.config import conf
        conf().removeConfigNotifier('services/http-transport/enabled')
        conf().removeConfigNotifier('services/http-transport/receiving-enabled')
        t = self.transport
        self.transport = None
        t.automat('shutdown')
        return True

    def _on_transport_state_changed(self, transport, oldstate, newstate):
        from logs import lg
        lg.info('%s -> %s in %r  starting_deferred=%r' % (oldstate, newstate, transport, bool(self.starting_deferred)))
        if self.starting_deferred:
            if newstate in ['LISTENING', ]:
                self.starting_deferred.callback(newstate)
                self.starting_deferred = None
            elif newstate in ['OFFLINE', ]:
                self.starting_deferred.errback(Exception(newstate))
                self.starting_deferred = None

    def _on_enabled_disabled(self, path, value, oldvalue, result):
        from p2p import network_connector
        from logs import lg
        lg.out(2, 'service_http_transport._on_enabled_disabled : %s->%s : %s' % (
            oldvalue, value, path))
        network_connector.A('reconnect')

    def _on_receiving_enabled_disabled(self, path, value, oldvalue, result):
        from p2p import network_connector
        from logs import lg
        lg.out(2, 'service_http_transport._on_receiving_enabled_disabled : %s->%s : %s' % (
            oldvalue, value, path))
        network_connector.A('reconnect')



#!/usr/bin/python
# service_customer_patrol.py
#
# Copyright (C) 2008 Veselin Penev, https://bitdust.io
#
# This file (service_customer_patrol.py) is part of BitDust Software.
#
# BitDust is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# BitDust Software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with BitDust Software.  If not, see <http://www.gnu.org/licenses/>.
#
# Please contact us if you have any questions at bitdust.io@gmail.com
#
#
#
#

"""
..

module:: service_customer_patrol
"""

from __future__ import absolute_import
from services.local_service import LocalService


def create_service():
    return CustomerPatrolService()


class CustomerPatrolService(LocalService):

    service_name = 'service_customer_patrol'
    config_path = 'services/customer-patrol/enabled'

    def dependent_on(self):
        return [
            'service_supplier',
        ]

    def start(self):
        from supplier import customers_rejector
        from main.config import conf
        from supplier import local_tester
        customers_rejector.A('restart')
        conf().addConfigNotifier('services/supplier/donated-space', self._on_donated_space_modified)
        local_tester.init()
        local_tester.start()
        return True

    def stop(self):
        from supplier import customers_rejector
        from main.config import conf
        from supplier import local_tester
        local_tester.stop()
        local_tester.shutdown()
        conf().removeConfigNotifier('services/supplier/donated-space')
        customers_rejector.Destroy()
        return True

    def health_check(self):
        return True

    def _on_donated_space_modified(self, path, value, oldvalue, result):
        from supplier import customers_rejector
        customers_rejector.A('restart')



#!/usr/bin/python
# api_web_socket.py
#
# Copyright (C) 2008 Veselin Penev, https://bitdust.io
#
# This file (api_web_socket.py) is part of BitDust Software.
#
# BitDust is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# BitDust Software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with BitDust Software.  If not, see <http://www.gnu.org/licenses/>.
#
# Please contact us if you have any questions at bitdust.io@gmail.com
#
#
#
#

"""
..

module:: api_web_socket
"""

#------------------------------------------------------------------------------

from __future__ import absolute_import

#------------------------------------------------------------------------------

_Debug = False
_DebugLevel = 10

_APILogFileEnabled = False

#------------------------------------------------------------------------------

from twisted.application.strports import listen
from twisted.internet.defer import Deferred
from twisted.internet.protocol import Protocol, Factory
from twisted.python.failure import Failure

#------------------------------------------------------------------------------

from logs import lg

from lib import txws
from lib import serialization

from system import local_fs

from main import events
from main import settings

from interface import api

#------------------------------------------------------------------------------

_WebSocketListener = None
_WebSocketTransports = {}
_AllAPIMethods = []
_APISecret = None

#------------------------------------------------------------------------------

def init(port=None):
    global _WebSocketListener
    global _AllAPIMethods
    global _APILogFileEnabled
    _APILogFileEnabled = settings.config.conf().getBool('logs/api-enabled')
    if _WebSocketListener is not None:
        lg.warn('_WebSocketListener already initialized')
        return
    if not port:
        port = settings.DefaultWebSocketPort()
    try:
        ws = BitDistWrappedWebSocketFactory(BitDustWebSocketFactory())
        _WebSocketListener = listen("tcp:%d" % port, ws)
    except:
        lg.exc()
        return
    _AllAPIMethods = set(dir(api))
    _AllAPIMethods.difference_update([
        # TODO: keep that list up to date when changing the api
        'on_api_result_prepared', 'Deferred', 'ERROR', 'Failure', 'OK', 'RESULT', '_Debug', '_DebugLevel',
        'strng', 'sys', 'time', 'gc', 'map', 'os',
        '__builtins__', '__cached__', '__doc__', '__file__', '__loader__', '__name__', '__package__', '__spec__',
        'absolute_import', 'driver', 'filemanager', 'jsn', 'lg',
        'event_listen', 'message_receive',
    ])
    if _Debug:
        lg.out(_DebugLevel, 'api_web_socket.init  _WebSocketListener=%r with %d methods' % (
            _WebSocketListener, len(_AllAPIMethods)))
    read_api_secret()
    events.add_subscriber(on_event, event_id='*')


def shutdown():
    global _WebSocketListener
    events.remove_subscriber(on_event, event_id='*')
    if _WebSocketListener:
        if _Debug:
            lg.out(_DebugLevel, 'api_web_socket.shutdown calling _WebSocketListener.stopListening()')
        _WebSocketListener.stopListening()
        del _WebSocketListener
        _WebSocketListener = None
        if _Debug:
            lg.out(_DebugLevel, '    _WebSocketListener destroyed')
    else:
        lg.warn('_WebSocketListener is None')

#------------------------------------------------------------------------------

def read_api_secret():
    global _APISecret
    _APISecret = local_fs.ReadTextFile(settings.APISecretFile())

#------------------------------------------------------------------------------

class BitDustWrappedWebSocketProtocol(txws.WebSocketProtocol):

    def validateHeaders(self):
        global _APISecret
        if _APISecret:
            _, _, api_secret_parameter = self.location.partition('?')
            _, _, api_secret_parameter = api_secret_parameter.partition('=')
            if api_secret_parameter != _APISecret:
                events.send('web-socket-access-denied', data=dict())
                self.loseConnection()
                return
        return txws.WebSocketProtocol.validateHeaders(self)


class BitDistWrappedWebSocketFactory(txws.WebSocketFactory):

    protocol = BitDustWrappedWebSocketProtocol


#------------------------------------------------------------------------------

class BitDustWebSocketProtocol(Protocol):

    _key = None

    def dataReceived(self, data):
        try:
            json_data = serialization.BytesToDict(data, keys_to_text=True, values_to_text=True, encoding='utf-8')
        except:
            lg.exc()
            return
        if _Debug:
            lg.dbg(_DebugLevel, 'received %d bytes from web socket: %r' % (len(data), json_data))
        if not do_process_incoming_message(json_data):
            lg.warn('failed processing incoming message from web socket: %r' % json_data)

    def connectionMade(self):
        global _WebSocketTransports
        Protocol.connectionMade(self)
        peer = self.transport.getPeer()
        self._key = (peer.type, peer.host, peer.port, )
        peer = '%s://%s:%s' % (self._key[0], self._key[1], self._key[2])
        _WebSocketTransports[self._key] = self.transport
        if _Debug:
            lg.args(_DebugLevel, key=self._key, ws_connections=len(_WebSocketTransports))
        events.send('web-socket-connected', data=dict(peer=peer))

    def connectionLost(self, *args, **kwargs):
        global _WebSocketTransports
        if _Debug:
            lg.args(_DebugLevel, key=self._key, ws_connections=len(_WebSocketTransports))
        Protocol.connectionLost(self, *args, **kwargs)
        _WebSocketTransports.pop(self._key)
        peer = '%s://%s:%s' % (self._key[0], self._key[1], self._key[2])
        self._key = None
        events.send('web-socket-disconnected', data=dict(peer=peer))

#------------------------------------------------------------------------------

class BitDustWebSocketFactory(Factory):

    protocol = BitDustWebSocketProtocol

    def buildProtocol(self, addr):
        """
        Only accepting connections from local machine!
        """
        if addr.host != '127.0.0.1':
            lg.err('refused connection from remote host: %r' % addr.host)
            return None
        proto = Factory.buildProtocol(self, addr)
        return proto

#------------------------------------------------------------------------------

def do_process_incoming_message(json_data):
    global _AllAPIMethods
    command = json_data.get('command')
    if command == 'api_call':
        method = json_data.get('method', None)
        kwargs = json_data.get('kwargs', {})
        call_id = json_data.get('call_id', None)

        if not method:
            lg.warn('api method name was not provided')
            return push({
                    'type': 'api_call',
                    'payload': {
                        'call_id': call_id,
                        'errors': ['api method name was not provided', ],
                    },
                })

        if method not in _AllAPIMethods:
            lg.warn('invalid api method name: %r' % method)
            return push({
                    'type': 'api_call',
                    'payload': {
                        'call_id': call_id,
                        'errors': ['invalid api method name', ],
                    },
                })

        if _Debug:
            lg.out(_DebugLevel, '*** %s  API WS IN  %s(%r)' % (
                call_id, method, kwargs))

        if _APILogFileEnabled:
            lg.out(0, '*** %s  WS IN  %s(%r)' % (
                call_id, method, kwargs), log_name='api', showtime=True)

        func = getattr(api, method)
        try:
            response = func(**kwargs)
        except Exception as err:
            lg.err(f'{method}({kwargs}) : {err}')
            return push({
                    'type': 'api_call',
                    'payload': {
                        'call_id': call_id,
                        'errors': [str(err), ],
                    },
                })

        if isinstance(response, Deferred):

            def _cb(r):
                return push({
                    'type': 'api_call',
                    'payload': {
                        'call_id': call_id,
                        'response': r,
                    },
                })

            def _eb(err):
                err_msg = err.getErrorMessage() if isinstance(err, Failure) else str(err)
                return push({
                    'type': 'api_call',
                    'payload': {
                        'call_id': call_id,
                        'errors': [err_msg, ],
                    },
                })

            response.addCallback(_cb)
            response.addErrback(_eb)
            return True

        return push({
            'type': 'api_call',
            'payload': {
                'call_id': call_id,
                'response': response,
            },
        })

    return False

#------------------------------------------------------------------------------

def on_event(evt):
    return push({
        'type': 'event',
        'payload': {
            'event_id': evt.event_id,
            'data': evt.data,
        },
    })


def on_stream_message(message_json):
    return push({
        'type': 'stream_message',
        'payload': message_json,
    })


def on_online_status_changed(status_info):
    return push({
        'type': 'online_status',
        'payload': status_info,
    })

#------------------------------------------------------------------------------

def push(json_data):
    global _WebSocketTransports
    if not _WebSocketTransports:
        return False
    raw_bytes = serialization.DictToBytes(json_data, encoding='utf-8')
    for _key, transp in _WebSocketTransports.items():
        transp.write(raw_bytes)
        if _Debug:
            lg.dbg(_DebugLevel, 'sent %d bytes to web socket %s' % (
                len(raw_bytes), '%s://%s:%s' % (_key[0], _key[1], _key[2]), ))
    if _Debug:
        lg.out(_DebugLevel, '***   API WS PUSH  %d bytes' % len(raw_bytes))
    if _APILogFileEnabled:
        lg.out(0, '*** WS PUSH  %d bytes : %r' % (len(raw_bytes), json_data, ), log_name='api', showtime=True)
    return True



from unittest import TestCase
import os

from system import bpio

from logs import lg

from main import settings

from lib import jsn
from lib import serialization

from crypt import key
from crypt import signed
from crypt import encrypted

from userid import my_id


_some_priv_key = """-----BEGIN RSA PRIVATE KEY-----
MIIEowIBAAKCAQEA/ZsJKyCakqA8vO2r0CTOG0qE2l+4y1dIqh7VC0oaVkXy0Cim
s3N+NNt6vJ617H5FeMALasRf4+5nHkleTB+BKTKBxt4fLm9Mz3mkw8U+pstJfYzS
FjjG3Zjs6XC4HyMjOCTkYCdvQQTeqB9ALAJ10YpVwlJlXOaFoscwv3LkUD4XB2Q4
rh6iqYdQo7Na2vY0aAw+t/wTnLTzr/rxQuhbng7ktPv8ZUx69eHzG07gLRpqzYH7
zkvuijtesxPIFFLaSlgQPc3rILqwLP/wI6fDjtPWlnG1cJJ9e7PBq1LkQ8FJoCxa
Sx9/zrZcVuKFnCPGrc1hjqCIU9Hh3wTPxV/PIQIDAQABAoIBAAO2fZXPXf33WTj2
SpI4e2PqRVEduXAQ2AIwGNQX4T70j8A5C82Tpg7ObLYxWruICCVjJo5Oin1vpIbi
Ac66im68yTTnnyRJajynyh1jivtV3XPYatdlvJvFePNMZHtbDYa4AQ0wlJwVRFB+
LSIxJXIO94/pz9y3uLAEINWZCt31xEpIWiJYDYvOnu/SUNJ652WzmEoguzdYndam
PjoC8S4PUWslCeMiHc5XLhrZVXoNFQyqAX/zZoKLtbNQq0Z3Huguih0ID5oSFWMl
MeFLUoW2UAvo+dwkW5DJc5VyBFvAX/J+03WqvqmX0198mD8LrMDr9hGsIAyQmGud
oECFIZECgYEA/hm8SiLaMjhvtvhPDYKG7+RbEomdf2F1sSrJtyUCTCswKpjGwDnP
zcxDJuGzIRYw16GoUBWIcJfrDXzzNN6TxX9qXhd8uJ+VckEp7H1mOYLnLN5Ekjyu
XqrYzG/cSRqb1IESG3ikm8/587WxRA+4/rm+XWg1XZCOotiJrCdz6LECgYEA/4Ba
auGkYErkw2E02WMRqo2Ct2cfWPCRXy/oWSvbDxNlhFoheKr/6e9PHQ+m5iLDo3sq
3ZaPgfDaty6uI4v/rCwDGdv2s5bkI9crZ2DxNKhgJUE9No87jSSHjBMv0eylUBHA
DB5n1jyEYFHTGpy4Hvr6M6sCz/8Ehvga9J596XECgYAl4F4xytotjD4Szxaxk9hb
X/W3YK4Kc9OgUhl5ZFngUru+TcGqm7N/IMiNviz+bJlhOyaksWECL5MJEqwKIHd0
hBat6eBcgOU8/7upFdQsFHgzNvqPtd3kHKFub+otN3stBQRW9ffLhgfjLR08YP+Z
cMSQld0GkmrAmXiEIelkMQKBgQCnyIUX3ReRuHzjpQkMnJc0Vft6LvkR8eC1DoPZ
UwhmrQkkUf/a+whVejaM0gN662doCvEKVN3mqeEnHDt00nHSgZCDwcQPCU7GDn3N
RIcBWnTQ4jethX/I3y04Gj1z8KBapV1lV+4+bL1Nd05XEoWCqrP1jB6rsj8p1vH0
o8PxsQKBgFxHBaLtYSqwQoolo0mmpGCHTdfmn+bRQaqL3DQEGYULtkj+gGuqVlj6
5cnFfTRgzj81rSkJIg7ZQ+V3CvtZSjYnflN2L1IS1aT+N9uvoLSV/yzNLmHlG6nR
TjpvKeAap/ydSo6j8K97O8h4EeVKM2xfbNBVpv1mpR8AbkJf7Ioz
-----END RSA PRIVATE KEY-----"""

_some_identity_xml = """<?xml version="1.0" encoding="utf-8"?>
<identity>
  <sources>
    <source>http://127.0.0.1:8084/alice.xml</source>
  </sources>
  <contacts>
    <contact>tcp://127.0.0.1:7103</contact>
  </contacts>
  <certificates/>
  <scrubbers/>
  <postage>0</postage>
  <date>Oct 06, 2018</date>
  <version></version>
  <revision>0</revision>
  <publickey>ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQD9mwkrIJqSoDy87avQJM4bSoTaX7jLV0iqHtULShpWRfLQKKazc34023q8nrXsfkV4wAtqxF/j7mceSV5MH4EpMoHG3h8ub0zPeaTDxT6my0l9jNIWOMbdmOzpcLgfIyM4JORgJ29BBN6oH0AsAnXRilXCUmVc5oWixzC/cuRQPhcHZDiuHqKph1Cjs1ra9jRoDD63/BOctPOv+vFC6FueDuS0+/xlTHr14fMbTuAtGmrNgfvOS+6KO16zE8gUUtpKWBA9zesgurAs//Ajp8OO09aWcbVwkn17s8GrUuRDwUmgLFpLH3/OtlxW4oWcI8atzWGOoIhT0eHfBM/FX88h</publickey>
  <signature>906441963064925827454594808955119786427327091644004203121255573673324339896720684156436969809331302081059171001468697744607176215971850395448358447221987030255681283469602485017439100048818715591137645334649289784704347547476744833593959051239987455656492991690929110057776187651745011046495572223232179906271885029639566994962961751420295596050170876718201976232073313442092469148257068924122205928495395843354406033087633663157320015417942886925426409641297819144861516715728591303834175552581452070473350075769438068777969720417764064190628218036180170101856467748070658740635441921977032005183554438715223073852</signature>
</identity>"""


class Test(TestCase):

    def setUp(self):
        try:
            bpio.rmdir_recursive('/tmp/.bitdust_tmp')
        except Exception:
            pass
        lg.set_debug_level(30)
        settings.init(base_dir='/tmp/.bitdust_tmp')
        self.my_current_key = None
        try:
            os.makedirs('/tmp/.bitdust_tmp/metadata/')
        except:
            pass
        try:
            os.makedirs('/tmp/.bitdust_tmp/logs/')
        except:
            pass
        fout = open('/tmp/_some_priv_key', 'w')
        fout.write(_some_priv_key)
        fout.close()
        fout = open(settings.LocalIdentityFilename(), 'w')
        fout.write(_some_identity_xml)
        fout.close()
        self.assertTrue(key.LoadMyKey(keyfilename='/tmp/_some_priv_key'))
        self.assertTrue(my_id.loadLocalIdentity())

    def tearDown(self):
        key.ForgetMyKey()
        my_id.forgetLocalIdentity()
        settings.shutdown()
        os.remove('/tmp/_some_priv_key')
        bpio.rmdir_recursive('/tmp/.bitdust_tmp')

    def test_jsn(self):
        data1 = os.urandom(1024)
        dct1 = {'d': {'data': data1, }, }
        raw = jsn.dumps(dct1, encoding='latin1')
        dct2 = jsn.loads(raw, encoding='latin1')
        data2 = dct2['d']['data']
        self.assertEqual(data1, data2)

    def test_serialization(self):
        data1 = os.urandom(1024)
        dct1 = {'d': {'data': data1, }, }
        raw = serialization.DictToBytes(dct1, encoding='latin1')
        dct2 = serialization.BytesToDict(raw, encoding='latin1')
        data2 = dct2['d']['data']
        self.assertEqual(data1, data2)

    def test_signed_packet(self):
        key.InitMyKey()
        data1 = os.urandom(1024)
        p1 = signed.Packet(
            'Data',
            my_id.getIDURL(),
            my_id.getIDURL(),
            'SomeID',
            data1,
            'RemoteID:abc',
        )
        self.assertTrue(p1.Valid())
        raw1 = p1.Serialize()

        p2 = signed.Unserialize(raw1)
        self.assertTrue(p2.Valid())
        raw2 = p2.Serialize()
        data2 = p2.Payload
        self.assertEqual(data1, data2)
        self.assertEqual(raw1, raw2)

    def test_encrypted_block(self):
        key.InitMyKey()
        data1 = os.urandom(1024)
        b1 = encrypted.Block(
            CreatorID=my_id.getIDURL(),
            BackupID='BackupABC',
            BlockNumber=123,
            SessionKey=key.NewSessionKey(session_key_type=key.SessionKeyType()),
            SessionKeyType=key.SessionKeyType(),
            LastBlock=True,
            Data=data1,
        )
        self.assertTrue(b1.Valid())
        raw1 = b1.Serialize()

        b2 = encrypted.Unserialize(raw1)
        self.assertTrue(b2.Valid())
        raw2 = b2.Serialize()
        data2 = b2.Data()
        self.assertEqual(data1, data2)
        self.assertEqual(raw1, raw2)



import os
import base64

from twisted.trial.unittest import TestCase
from twisted.internet import reactor  # @UnresolvedImport
from twisted.internet.defer import Deferred
from twisted.internet.base import DelayedCall
DelayedCall.debug = True


from raid import raid_worker

from logs import lg

from system import bpio

from main import settings


class TestRaidWorker(TestCase):

    def setUp(self):
        try:
            bpio.rmdir_recursive('/tmp/.bitdust_tmp')
        except Exception:
            pass
        settings.init(base_dir='/tmp/.bitdust_tmp')
        lg.set_debug_level(30)
        try:
            os.makedirs('/tmp/.bitdust_tmp/logs')
        except:
            pass

    def tearDown(self):
        settings.shutdown()
        bpio.rmdir_recursive('/tmp/.bitdust_tmp')

    def _test_make_rebuild_read(self, target_ecc_map, num_suppliers, dead_suppliers, read_success, rebuild_one_success, filesize, try_rebuild=False):
        test_result = Deferred()

        curdir = os.getcwd()
        if curdir.count('_trial_temp'):
            os.chdir(os.path.abspath(os.path.join(curdir, '..')))

        def _read_done(cmd, taskdata, result):
            print('_read_done', cmd, taskdata, result)
            final_result = False
            source_data = open('/tmp/source.txt', 'rb').read()
            reconstructed_data = open('/tmp/destination.txt', 'rb').read()
            if read_success:
                final_result = (source_data == reconstructed_data)
            else:
                final_result = (source_data != reconstructed_data)
            os.system('rm -rf /tmp/source.txt')
            os.system('rm -rf /tmp/destination.txt')
            os.system('rm -rf /tmp/raidtest')
            reactor.callLater(0, raid_worker.A, 'shutdown')  # @UndefinedVariable
            if final_result:
                reactor.callLater(0.1, test_result.callback, True)  # @UndefinedVariable
            else:
                print(source_data)
                print(reconstructed_data)
                if read_success:
                    reactor.callLater(0.1, test_result.errback, Exception('reconstructed data is not the same as source data'))  # @UndefinedVariable
                else:
                    reactor.callLater(0.1, test_result.errback, Exception('reconstructed data is the same as source data, but expect to fail the raid read'))  # @UndefinedVariable
            return True

        def _rebuild_done(cmd, taskdata, result):
            newData = result[0]
            if rebuild_one_success:
                if not newData:
                    os.system('rm -rf /tmp/source.txt')
                    os.system('rm -rf /tmp/raidtest')
                    reactor.callLater(0, raid_worker.A, 'shutdown')  # @UndefinedVariable
                    reactor.callLater(0.1, test_result.errback, Exception('rebuild expected to succeed but new data was not created'))  # @UndefinedVariable
                    return
            else:
                if newData:
                    os.system('rm -rf /tmp/source.txt')
                    os.system('rm -rf /tmp/raidtest')
                    reactor.callLater(0, raid_worker.A, 'shutdown')  # @UndefinedVariable
                    reactor.callLater(0.1, test_result.errback, Exception('rebuild expected to fail but new data was created'))  # @UndefinedVariable
                    return
            # try to read all fragments now
            reactor.callLater(0.5, raid_worker.add_task, 'read', (  # @UndefinedVariable
                '/tmp/destination.txt', target_ecc_map, 'F12345678', '5', '/tmp/raidtest/master$alice@somehost.com/0'), _read_done)
            return True

        def _make_done(cmd, taskdata, result):
            if list(result) != [num_suppliers, num_suppliers]:
                os.system('rm -rf /tmp/source.txt')
                os.system('rm -rf /tmp/raidtest')
                reactor.callLater(0, raid_worker.A, 'shutdown')  # @UndefinedVariable
                reactor.callLater(0.1, test_result.callback, False)  # @UndefinedVariable
                return
            # remove few fragments and try to rebuild the whole block
            for supplier_position in range(dead_suppliers):
                os.system("rm -rf '/tmp/raidtest/master$alice@somehost.com/0/F12345678/5-%d-Data'" % supplier_position)
                os.system("rm -rf '/tmp/raidtest/master$alice@somehost.com/0/F12345678/5-%d-Parity'" % supplier_position)
            alive_suppliers = num_suppliers - dead_suppliers
            remote_fragments = {
                'D': [0, ] * dead_suppliers + [1, ] * alive_suppliers,
                'P': [0, ] * dead_suppliers + [1, ] * alive_suppliers,
            }
            local_fragments = {
                'D': [0, ] * dead_suppliers + [1, ] * alive_suppliers,
                'P': [0, ] * dead_suppliers + [1, ] * alive_suppliers,
            }
            if not try_rebuild:
                reactor.callLater(0.5, raid_worker.add_task, 'read', (  # @UndefinedVariable
                    '/tmp/destination.txt', target_ecc_map, 'F12345678', '5', '/tmp/raidtest/master$alice@somehost.com/0'), _read_done)
            else:
                reactor.callLater(0.5, raid_worker.add_task, 'rebuild', (  # @UndefinedVariable
                    'master$alice@somehost.com:0/F12345678', '5', target_ecc_map, [1, ] * num_suppliers,
                    remote_fragments, local_fragments, '/tmp/raidtest'), _rebuild_done)
            return True

        os.system('rm -rf /tmp/source.txt')
        os.system('rm -rf /tmp/destination.txt')
        os.system('rm -rf /tmp/raidtest')
        os.system("mkdir -p '/tmp/raidtest/master$alice@somehost.com/0/F12345678'")
        open('/tmp/source.txt', 'wb').write(base64.b64encode(os.urandom(filesize)))
        reactor.callWhenRunning(raid_worker.A, 'init')  # @UndefinedVariable
        reactor.callLater(0.5, raid_worker.add_task, 'make', (  # @UndefinedVariable
            '/tmp/source.txt', target_ecc_map, 'F12345678', '5', '/tmp/raidtest/master$alice@somehost.com/0/F12345678'), _make_done)
        return test_result


    def test_ecc2x2_with_0_dead_suppliers_success(self):
        return self._test_make_rebuild_read(
            target_ecc_map='ecc/2x2',
            num_suppliers=2,
            dead_suppliers=0,
            rebuild_one_success=False,
            read_success=True,
            filesize=50,
            try_rebuild=True,
        )

    def test_ecc2x2_with_1_dead_supplier_success_no_rebuild(self):
        return self._test_make_rebuild_read(
            target_ecc_map='ecc/2x2',
            num_suppliers=2,
            dead_suppliers=1,
            rebuild_one_success=True,
            read_success=True,
            filesize=50,
            try_rebuild=False,
        )

    def test_ecc2x2_with_1_dead_supplier_success(self):
        return self._test_make_rebuild_read(
            target_ecc_map='ecc/2x2',
            num_suppliers=2,
            dead_suppliers=1,
            rebuild_one_success=True,
            read_success=True,
            filesize=50,
            try_rebuild=False,
        )

    def test_ecc2x2_with_2_dead_suppliers_failed(self):
        return self._test_make_rebuild_read(
            target_ecc_map='ecc/2x2',
            num_suppliers=2,
            dead_suppliers=2,
            rebuild_one_success=False,
            read_success=False,
            filesize=50,
            try_rebuild=True,
        )

    def test_ecc4x4_with_2_dead_suppliers_success(self):
        return self._test_make_rebuild_read(
            target_ecc_map='ecc/4x4',
            num_suppliers=4,
            dead_suppliers=2,
            rebuild_one_success=True,
            read_success=True,
            filesize=50,
            try_rebuild=True,
        )

    def test_ecc4x4_with_3_dead_suppliers_failed(self):
        return self._test_make_rebuild_read(
            target_ecc_map='ecc/4x4',
            num_suppliers=4,
            dead_suppliers=3,
            rebuild_one_success=False,
            read_success=False,
            filesize=50,
            try_rebuild=True,
        )

    def test_ecc4x4_with_2_dead_suppliers_success_no_rebuild(self):
        return self._test_make_rebuild_read(
            target_ecc_map='ecc/4x4',
            num_suppliers=4,
            dead_suppliers=2,
            rebuild_one_success=True,
            read_success=True,
            filesize=50,
            try_rebuild=False,
        )

    def test_ecc7x7_with_4_dead_suppliers_failed(self):
        return self._test_make_rebuild_read(
            target_ecc_map='ecc/7x7',
            num_suppliers=7,
            dead_suppliers=4,
            rebuild_one_success=False,
            read_success=False,
            filesize=50,
            try_rebuild=True,
        )

    def test_ecc18x18_with_5_dead_suppliers_success(self):
        return self._test_make_rebuild_read(
            target_ecc_map='ecc/18x18',
            num_suppliers=18,
            dead_suppliers=5,
            rebuild_one_success=True,
            read_success=True,
            filesize=50,
            try_rebuild=True,
        )

    def test_ecc18x18_with_10_dead_suppliers_failed(self):
        return self._test_make_rebuild_read(
            target_ecc_map='ecc/18x18',
            num_suppliers=18,
            dead_suppliers=10,  # TODO: strange, for 18 suppliers max 5 "correctable" errors are possible, see raid/eccmap.py
            rebuild_one_success=True,
            read_success=False,
            filesize=50,
            try_rebuild=True,
        )

    def test_ecc18x18_with_14_dead_suppliers_failed(self):
        return self._test_make_rebuild_read(
            target_ecc_map='ecc/18x18',
            num_suppliers=18,
            dead_suppliers=14,
            rebuild_one_success=False,
            read_success=False,
            filesize=50,
            try_rebuild=True,
        )

    def test_ecc64x64_with_10_dead_suppliers_success(self):
        return self._test_make_rebuild_read(
            target_ecc_map='ecc/64x64',
            num_suppliers=64,
            dead_suppliers=10,
            rebuild_one_success=True,
            read_success=True,
            filesize=50,
            try_rebuild=True,
        )

    def test_ecc64x64_with_10_dead_suppliers_success_no_rebuild(self):
        return self._test_make_rebuild_read(
            target_ecc_map='ecc/64x64',
            num_suppliers=64,
            dead_suppliers=10,
            rebuild_one_success=True,
            read_success=True,
            filesize=50,
            try_rebuild=False,
        )

    def test_ecc64x64_with_23_dead_suppliers_failed(self):
        return self._test_make_rebuild_read(
            target_ecc_map='ecc/64x64',
            num_suppliers=64,
            dead_suppliers=23,  # TODO: strange, for 64 suppliers max 10 "correctable" errors are possible, see raid/eccmap.py
            rebuild_one_success=True,
            read_success=False,
            filesize=50,
            try_rebuild=True,
        )

    def test_task_cancel(self):
        test_result = Deferred()
        os.system('rm -rf /tmp/source.txt')
        os.system('rm -rf /tmp/destination.txt')
        os.system('rm -rf /tmp/raidtest')
        os.system("mkdir -p '/tmp/raidtest/master$alice@somehost.com/0/F12345678'")
        open('/tmp/source1.txt', 'w').write(base64.b64encode(os.urandom(1000000)).decode())
        reactor.callWhenRunning(raid_worker.A, 'init')  # @UndefinedVariable

        def _task_failed(c, t, r):
            final_result = (r == (-1, -1) or r is None)
            os.system('rm -rf /tmp/source1.txt')
            os.system('rm -rf /tmp/raidtest')
            reactor.callLater(0, raid_worker.A, 'shutdown')  # @UndefinedVariable
            if final_result:
                reactor.callLater(0.1, test_result.callback, True)  # @UndefinedVariable
            else:
                reactor.callLater(0.1, test_result.errback, Exception('task expected to fail, but positive result was returned'))  # @UndefinedVariable

        reactor.callLater(0.5, raid_worker.add_task, 'make', (  # @UndefinedVariable
            '/tmp/source1.txt', 'ecc/64x64', 'F12345678', '5', '/tmp/raidtest/master$alice@somehost.com/0/F12345678'), _task_failed)
        reactor.callLater(0.55, raid_worker.cancel_task, 'make', '/tmp/source1.txt')  # @UndefinedVariable

        return test_result



#!/usr/bin/python
# config_defaults.py
#
# Copyright (C) 2008 Veselin Penev, https://bitdust.io
#
# This file (config_defaults.py) is part of BitDust Software.
#
# BitDust is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# BitDust Software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with BitDust Software.  If not, see <http://www.gnu.org/licenses/>.
#
# Please contact us if you have any questions at bitdust.io@gmail.com
#
#
#
#

"""
..

module:: config_defaults
"""


def reset(conf_obj):
    """
    Configure default values for all BitDust local settings inside ~/.bitdust/config/ folder.

    Every option must have a default value, however there are exceptions possible.

    Here all items suppose to have some widget in UI to interact with human.
    Keep it simple and understandable.
    """
    from lib import diskspace
    from main import settings

    conf_obj.setDefaultValue('interface/api/rest-http-enabled', 'true')
    conf_obj.setDefaultValue('interface/api/rest-http-port', settings.DefaultRESTHTTPPort())

    conf_obj.setDefaultValue('interface/api/web-socket-enabled', 'true')
    conf_obj.setDefaultValue('interface/api/web-socket-port', settings.DefaultWebSocketPort())

    conf_obj.setDefaultValue('interface/ftp/enabled', 'true')
    conf_obj.setDefaultValue('interface/ftp/port', settings.DefaultFTPPort())

    conf_obj.setDefaultValue('logs/api-enabled', 'false')
    conf_obj.setDefaultValue('logs/automat-transitions-enabled', 'false')
    conf_obj.setDefaultValue('logs/automat-events-enabled', 'false')
    conf_obj.setDefaultValue('logs/debug-level', settings.defaultDebugLevel())
    conf_obj.setDefaultValue('logs/memdebug-enabled', 'false')
    conf_obj.setDefaultValue('logs/memdebug-port', '9996')
    conf_obj.setDefaultValue('logs/memprofile-enabled', 'false')
    conf_obj.setDefaultValue('logs/packet-enabled', 'false')
    conf_obj.setDefaultValue('logs/stream-enabled', 'false')
    conf_obj.setDefaultValue('logs/stream-port', settings.DefaultWebLogPort())
    conf_obj.setDefaultValue('logs/traffic-enabled', 'false')
    conf_obj.setDefaultValue('logs/traffic-port', settings.DefaultWebTrafficPort())

    conf_obj.setDefaultValue('paths/backups', '')
    conf_obj.setDefaultValue('paths/customers', '')
    conf_obj.setDefaultValue('paths/messages', '')
    conf_obj.setDefaultValue('paths/receipts', '')
    conf_obj.setDefaultValue('paths/restore', '')

    conf_obj.setDefaultValue('personal/private-key-size', settings.DefaultPrivateKeySize())

    conf_obj.setDefaultValue('services/accountant/enabled', 'false')

    conf_obj.setDefaultValue('services/backup-db/enabled', 'true')

    conf_obj.setDefaultValue('services/backups/enabled', 'true')
    conf_obj.setDefaultValue('services/backups/block-size', diskspace.MakeStringFromBytes(settings.DefaultBackupBlockSize()))
    conf_obj.setDefaultValue('services/backups/max-block-size', diskspace.MakeStringFromBytes(settings.DefaultBackupMaxBlockSize()))
    conf_obj.setDefaultValue('services/backups/max-copies', '2')
    conf_obj.setDefaultValue('services/backups/keep-local-copies-enabled', 'true')
    conf_obj.setDefaultValue('services/backups/wait-suppliers-enabled', 'true')

    conf_obj.setDefaultValue('services/blockchain/enabled', 'false')
    conf_obj.setDefaultValue('services/blockchain/host', '127.0.0.1')
    conf_obj.setDefaultValue('services/blockchain/port', 9100)
    conf_obj.setDefaultValue('services/blockchain/seeds', '')
    conf_obj.setDefaultValue('services/blockchain/explorer/enabled', 'true')
    conf_obj.setDefaultValue('services/blockchain/explorer/port', 9180)
    conf_obj.setDefaultValue('services/blockchain/wallet/enabled', 'true')
    conf_obj.setDefaultValue('services/blockchain/wallet/port', 9280)
    conf_obj.setDefaultValue('services/blockchain/miner/enabled', 'false')

    conf_obj.setDefaultValue('services/broadcasting/enabled', 'false')
    conf_obj.setDefaultValue('services/broadcasting/routing-enabled', 'false')
    conf_obj.setDefaultValue('services/broadcasting/max-broadcast-connections', '3')

    conf_obj.setDefaultValue('services/contract-chain/enabled', 'false')

    conf_obj.setDefaultValue('services/customer/enabled', 'true')
    conf_obj.setDefaultValue('services/customer/needed-space', diskspace.MakeStringFromBytes(settings.DefaultNeededBytes()))
    conf_obj.setDefaultValue('services/customer/suppliers-number', settings.DefaultDesiredSuppliers())

    conf_obj.setDefaultValue('services/customer-contracts/enabled', 'false')

    conf_obj.setDefaultValue('services/customer-family/enabled', 'true')

    conf_obj.setDefaultValue('services/customer-patrol/enabled', 'true')
    conf_obj.setDefaultValue('services/customer-patrol/customer-idle-days', 14)

    conf_obj.setDefaultValue('services/customer-support/enabled', 'true')

    conf_obj.setDefaultValue('services/data-disintegration/enabled', 'true')

    conf_obj.setDefaultValue('services/data-motion/enabled', 'true')
    conf_obj.setDefaultValue('services/data-motion/supplier-request-queue-size', 4)
    conf_obj.setDefaultValue('services/data-motion/supplier-sending-queue-size', 4)

    conf_obj.setDefaultValue('services/entangled-dht/enabled', 'true')
    conf_obj.setDefaultValue('services/entangled-dht/udp-port', settings.DefaultDHTPort())
    conf_obj.setDefaultValue('services/entangled-dht/known-nodes', '')
    conf_obj.setDefaultValue('services/entangled-dht/attached-layers', '')

    conf_obj.setDefaultValue('services/employer/enabled', 'true')
    conf_obj.setDefaultValue('services/employer/replace-critically-offline-enabled', 'true')
    conf_obj.setDefaultValue('services/employer/candidates', '')

    conf_obj.setDefaultValue('services/gateway/enabled', 'true')

    conf_obj.setDefaultValue('services/http-connections/enabled', 'false')
    conf_obj.setDefaultValue('services/http-connections/http-port', settings.DefaultHTTPPort())

    conf_obj.setDefaultValue('services/http-transport/enabled', 'false')  # not done yet
    conf_obj.setDefaultValue('services/http-transport/receiving-enabled', 'true')
    conf_obj.setDefaultValue('services/http-transport/sending-enabled', 'true')
    conf_obj.setDefaultValue('services/http-transport/priority', 50)

    conf_obj.setDefaultValue('services/identity-server/enabled', 'false')
    conf_obj.setDefaultValue('services/identity-server/host', '')
    conf_obj.setDefaultValue('services/identity-server/tcp-port', settings.IdentityServerPort())
    conf_obj.setDefaultValue('services/identity-server/web-port', settings.IdentityWebPort())

    conf_obj.setDefaultValue('services/identity-propagate/enabled', 'true')
    conf_obj.setDefaultValue('services/identity-propagate/known-servers', '')
    conf_obj.setDefaultValue('services/identity-propagate/preferred-servers', '')
    conf_obj.setDefaultValue('services/identity-propagate/min-servers', 2)
    conf_obj.setDefaultValue('services/identity-propagate/max-servers', 5)
    conf_obj.setDefaultValue('services/identity-propagate/automatic-rotate-enabled', 'true')
    conf_obj.setDefaultValue('services/identity-propagate/health-check-interval-seconds', 60*5)

    conf_obj.setDefaultValue('services/ip-port-responder/enabled', 'true')

    conf_obj.setDefaultValue('services/keys-registry/enabled', 'true')

    conf_obj.setDefaultValue('services/keys-storage/enabled', 'true')
    conf_obj.setDefaultValue('services/keys-storage/reset-unreliable-backup-copies', 'true')

    conf_obj.setDefaultValue('services/list-files/enabled', 'true')

    conf_obj.setDefaultValue('services/message-broker/enabled', 'false')
    conf_obj.setDefaultValue('services/message-broker/archive-chunk-size', 100)
    conf_obj.setDefaultValue('services/message-broker/message-ack-timeout', 30)
    conf_obj.setDefaultValue('services/message-broker/broker-negotiate-ack-timeout', 30)
    conf_obj.setDefaultValue('services/message-broker/preferred-brokers', '')

    conf_obj.setDefaultValue('services/message-history/enabled', 'true')

    conf_obj.setDefaultValue('services/miner/enabled', 'false')

    conf_obj.setDefaultValue('services/my-data/enabled', 'true')

    conf_obj.setDefaultValue('services/my-ip-port/enabled', 'true')

    conf_obj.setDefaultValue('services/network/enabled', 'true')
    conf_obj.setDefaultValue('services/network/proxy/enabled', 'false')
    conf_obj.setDefaultValue('services/network/proxy/host', '')
    conf_obj.setDefaultValue('services/network/proxy/password', '')
    conf_obj.setDefaultValue('services/network/proxy/port', '')
    conf_obj.setDefaultValue('services/network/proxy/ssl', 'false')
    conf_obj.setDefaultValue('services/network/proxy/username', '')
    conf_obj.setDefaultValue('services/network/receive-limit', settings.DefaultBandwidthInLimit())
    conf_obj.setDefaultValue('services/network/send-limit', settings.DefaultBandwidthOutLimit())

    conf_obj.setDefaultValue('services/nodes-lookup/enabled', 'true')

    conf_obj.setDefaultValue('services/p2p-hookups/enabled', 'true')

    conf_obj.setDefaultValue('services/p2p-notifications/enabled', 'true')

    conf_obj.setDefaultValue('services/personal-messages/enabled', 'true')

    conf_obj.setDefaultValue('services/private-groups/enabled', 'true')
    conf_obj.setDefaultValue('services/private-groups/message-ack-timeout', 30)
    conf_obj.setDefaultValue('services/private-groups/broker-connect-timeout', 120)
    conf_obj.setDefaultValue('services/private-groups/preferred-brokers', '')

    conf_obj.setDefaultValue('services/private-messages/enabled', 'true')
    conf_obj.setDefaultValue('services/private-messages/acknowledge-unread-messages-enabled', 'true')

    conf_obj.setDefaultValue('services/proxy-server/enabled', 'false')
    conf_obj.setDefaultValue('services/proxy-server/routes-limit', 20)
    conf_obj.setDefaultValue('services/proxy-server/current-routes', '{}')

    conf_obj.setDefaultValue('services/proxy-transport/enabled', 'true')
    conf_obj.setDefaultValue('services/proxy-transport/sending-enabled', 'true')
    conf_obj.setDefaultValue('services/proxy-transport/receiving-enabled', 'true')
    conf_obj.setDefaultValue('services/proxy-transport/priority', 100)
    conf_obj.setDefaultValue('services/proxy-transport/preferred-routers', '')
    # conf_obj.setDefaultValue('services/proxy-transport/router-lifetime-seconds', 600)
    # TODO: those two settings needs to be removed.
    # if service require storing locally value which user should not modify be implemented
    # inside service (for example read/write to local file inside ~/.bitdust/*/ folder)
    # in the future we can split all files more structural way into ~/.bitdust/services/*/ sub folders
    conf_obj.setDefaultValue('services/proxy-transport/my-original-identity', '')
    conf_obj.setDefaultValue('services/proxy-transport/current-router', '')

    conf_obj.setDefaultValue('services/rebuilding/enabled', 'true')

    conf_obj.setDefaultValue('services/restores/enabled', 'true')

    conf_obj.setDefaultValue('services/shared-data/enabled', 'true')

    conf_obj.setDefaultValue('services/supplier/enabled', 'true')
    conf_obj.setDefaultValue('services/supplier/donated-space', diskspace.MakeStringFromBytes(settings.DefaultDonatedBytes()))

    conf_obj.setDefaultValue('services/supplier-contracts/enabled', 'false')

    conf_obj.setDefaultValue('services/tcp-connections/enabled', 'true')
    conf_obj.setDefaultValue('services/tcp-connections/tcp-port', settings.DefaultTCPPort())
    conf_obj.setDefaultValue('services/tcp-connections/upnp-enabled', 'true')

    conf_obj.setDefaultValue('services/tcp-transport/enabled', 'true')
    conf_obj.setDefaultValue('services/tcp-transport/receiving-enabled', 'true')
    conf_obj.setDefaultValue('services/tcp-transport/sending-enabled', 'true')
    conf_obj.setDefaultValue('services/tcp-transport/priority', 10)

    conf_obj.setDefaultValue('services/udp-datagrams/enabled', 'true')
    conf_obj.setDefaultValue('services/udp-datagrams/udp-port', settings.DefaultUDPPort())

    # TODO: UDP transport was temporary switched off
    conf_obj.setDefaultValue('services/udp-transport/enabled', 'false')
    conf_obj.setDefaultValue('services/udp-transport/receiving-enabled', 'true')
    conf_obj.setDefaultValue('services/udp-transport/sending-enabled', 'true')
    conf_obj.setDefaultValue('services/udp-transport/priority', 20)

